package gregapi.render;

import static gregapi.data.CS.F;
import static gregapi.data.CS.T;
import static gregapi.data.CS.UNCOLOURED;
import gregapi.GT_API;
import gregapi.code.ArrayListNoNulls;
import gregapi.data.CS.ModIDs;

import java.util.List;

import net.minecraft.client.renderer.texture.IIconRegister;
import net.minecraft.client.renderer.texture.TextureMap;
import net.minecraft.util.IIcon;
import net.minecraft.util.ResourceLocation;

/** 
 * @author Gregorius Techneticies
 */
public class TextureSet {
	public static final List<TextureSet> INSTANCES_ITEM = new ArrayListNoNulls<TextureSet>();
	public static final List<TextureSet> INSTANCES_BLOCK = new ArrayListNoNulls<TextureSet>();
	public static final List<String> FILENAMES_ITEM = new ArrayListNoNulls<String>();
	public static final List<String> FILENAMES_BLOCK = new ArrayListNoNulls<String>();
	public final List<IIconContainer> mList = new ArrayListNoNulls<IIconContainer>();
	
	private final boolean mIsItem;
	private final String mNameSet;
	
	private TextureSet(boolean aIsItem, String aNameSet) {
		mIsItem = aIsItem;
		mNameSet = aNameSet;
	}
	
	/**
	 * Adds a new Texture Set, unless another Set with the same Name already exists.
	 * @param aModID the Mod responsible for adding the Icons (which are located in either assets\aModID.toLowerCase()\textures\items\materialicons\ or assets\aModID.toLowerCase()\textures\blocks\materialicons\).
	 * @param aIsItem true if this is an Item Icon, false if this is a Block Icon.
	 */
	public static TextureSet addTextureSet(String aModID, boolean aIsItem, String aNameSet) {
		List<TextureSet> tTextureSetList = (aIsItem?INSTANCES_ITEM:INSTANCES_BLOCK);
		List<String> tFileNameList = (aIsItem?FILENAMES_ITEM:FILENAMES_BLOCK);
		for (TextureSet tSet : tTextureSetList) if (tSet.mNameSet.equals(aNameSet)) return tSet;
		TextureSet rSet = new TextureSet(aIsItem, aNameSet);
		tTextureSetList.add(rSet);
		for (int i = 0, j = tFileNameList.size(); i < j; i++) rSet.mList.add(aIsItem?new TextureSetIconItem(aModID, aNameSet+"/"+tFileNameList.get(i)):new TextureSetIconBlock(aModID, aNameSet+"/"+tFileNameList.get(i)));
		return rSet;
	}
	
	/**
	 * Adds a new Icon Name to all Texture Sets, or just returns the Index of an already existing Set with the same File Name.
	 * @param aModID the Mod responsible for adding the Icons (which are located in either assets\aModID.toLowerCase()\textures\items\materialicons\ or assets\aModID.toLowerCase()\textures\blocks\materialicons\).
	 * @param aNameFile the Name of the File inside each Set Folder.
	 * @return the Index of this Icon inside the Set Lists.
	 */
	public static int addToAll(String aModID, boolean aIsItem, String aNameFile) {
		List<String> tFileNameList = (aIsItem?FILENAMES_ITEM:FILENAMES_BLOCK);
		int tIndex = tFileNameList.indexOf(aNameFile);
		if (tIndex >= 0) return tIndex;
		tFileNameList.add(aNameFile);
		for (TextureSet tSet : (aIsItem?INSTANCES_ITEM:INSTANCES_BLOCK)) tSet.mList.add(tSet.mIsItem?new TextureSetIconItem(aModID, tSet.mNameSet+"/"+aNameFile):new TextureSetIconBlock(aModID, tSet.mNameSet+"/"+aNameFile));
		return tFileNameList.size() - 1;
	}
	
	public static class TextureSetIconItem implements IIconContainer, Runnable {
		private final String mMod, mName;
		private IIcon mIconColored, mIconOverlay;
		
		public TextureSetIconItem(String aMod, String aName) {
			mName = aName;
			mMod = aMod.toLowerCase();
			if (GT_API.sItemIconload != null) GT_API.sItemIconload.add(this);
		}
		
		@Override
		public IIcon getIcon(int aRenderPass) {
			return aRenderPass == 0 ? mIconColored : mIconOverlay;
		}
		
		@Override
		public short[] getIconColor(int aRenderPass) {
			return UNCOLOURED;
		}
		
		@Override
		public int getIconPasses() {
			return 2;
		}
		
		@Override
		public ResourceLocation getTextureFile() {
			return TextureMap.locationItemsTexture;
		}
		
		@Override
		public void registerIcons(IIconRegister aIconRegister) {
			mIconColored = aIconRegister.registerIcon(mMod+":materialicons/"+mName);
			mIconOverlay = aIconRegister.registerIcon(mMod+":materialicons/"+mName+"_OVERLAY");
		}
		
		@Override
		public String toString() {
			return mMod+":materialicons/"+mName;
		}
		
		@Override
		public boolean isUsingColorModulation(int aRenderPass) {
			return aRenderPass == 0;
		}
		
		@Override
		public void run() {
			registerIcons(GT_API.sItemIcons);
		}
	}
	
	public static class TextureSetIconBlock implements IIconContainer, Runnable {
		private final String mMod, mName;
		private IIcon mIconColored, mIconOverlay;
		
		public TextureSetIconBlock(String aMod, String aName) {
			mName = aName;
			mMod = aMod;
			if (GT_API.sBlockIconload != null) GT_API.sBlockIconload.add(this);
		}
		
		@Override
		public IIcon getIcon(int aRenderPass) {
			return aRenderPass == 0 ? mIconColored : mIconOverlay;
		}
		
		@Override
		public short[] getIconColor(int aRenderPass) {
			return UNCOLOURED;
		}
		
		@Override
		public int getIconPasses() {
			return 2;
		}
		
		@Override
		public ResourceLocation getTextureFile() {
			return TextureMap.locationBlocksTexture;
		}
		
		@Override
		public void registerIcons(IIconRegister aIconRegister) {
			mIconColored = aIconRegister.registerIcon(mMod+":materialicons/"+mName);
			mIconOverlay = aIconRegister.registerIcon(mMod+":materialicons/"+mName+"_OVERLAY");
		}
		
		@Override
		public String toString() {
			return mMod+":materialicons/"+mName;
		}
		
		@Override
		public boolean isUsingColorModulation(int aRenderPass) {
			return aRenderPass == 0;
		}
		
		@Override
		public void run() {
			registerIcons(GT_API.sBlockIcons);
		}
	}
	
	/** Arrays with all Sets, which belong to each other. Shortens the Parameter amount inside MT.java */
	public static final TextureSet[]
	  SET_NONE						= new TextureSet[] {addTextureSet(ModIDs.GT, F, "NONE")          , addTextureSet(ModIDs.GT, T, "NONE")          }
	, SET_DULL						= new TextureSet[] {addTextureSet(ModIDs.GT, F, "DULL")          , addTextureSet(ModIDs.GT, T, "DULL")          }
	, SET_RUBY						= new TextureSet[] {addTextureSet(ModIDs.GT, F, "RUBY")          , addTextureSet(ModIDs.GT, T, "RUBY")          }
	, SET_OPAL						= new TextureSet[] {addTextureSet(ModIDs.GT, F, "OPAL")          , addTextureSet(ModIDs.GT, T, "OPAL")          }
	, SET_LEAF						= new TextureSet[] {addTextureSet(ModIDs.GT, F, "LEAF")          , addTextureSet(ModIDs.GT, T, "LEAF")          }
	, SET_SAND						= new TextureSet[] {addTextureSet(ModIDs.GT, F, "SAND")          , addTextureSet(ModIDs.GT, T, "SAND")          }
	, SET_FINE						= new TextureSet[] {addTextureSet(ModIDs.GT, F, "FINE")          , addTextureSet(ModIDs.GT, T, "FINE")          }
	, SET_FOOD						= new TextureSet[] {addTextureSet(ModIDs.GT, F, "FOOD")          , addTextureSet(ModIDs.GT, T, "FOOD")          }
	, SET_WOOD						= new TextureSet[] {addTextureSet(ModIDs.GT, F, "WOOD")          , addTextureSet(ModIDs.GT, T, "WOOD")          }
	, SET_CUBE						= new TextureSet[] {addTextureSet(ModIDs.GT, F, "CUBE")          , addTextureSet(ModIDs.GT, T, "CUBE")          }
	, SET_FIERY						= new TextureSet[] {addTextureSet(ModIDs.GT, F, "FIERY")         , addTextureSet(ModIDs.GT, T, "FIERY")         }
	, SET_FLUID						= new TextureSet[] {addTextureSet(ModIDs.GT, F, "FLUID")         , addTextureSet(ModIDs.GT, T, "FLUID")         }
	, SET_ROUGH						= new TextureSet[] {addTextureSet(ModIDs.GT, F, "ROUGH")         , addTextureSet(ModIDs.GT, T, "ROUGH")         }
	, SET_STONE						= new TextureSet[] {addTextureSet(ModIDs.GT, F, "STONE")         , addTextureSet(ModIDs.GT, T, "STONE")         }
	, SET_PAPER						= new TextureSet[] {addTextureSet(ModIDs.GT, F, "PAPER")         , addTextureSet(ModIDs.GT, T, "PAPER")         }
	, SET_GLASS						= new TextureSet[] {addTextureSet(ModIDs.GT, F, "GLASS")         , addTextureSet(ModIDs.GT, T, "GLASS")         }
	, SET_FLINT						= new TextureSet[] {addTextureSet(ModIDs.GT, F, "FLINT")         , addTextureSet(ModIDs.GT, T, "FLINT")         }
	, SET_LAPIS						= new TextureSet[] {addTextureSet(ModIDs.GT, F, "LAPIS")         , addTextureSet(ModIDs.GT, T, "LAPIS")         }
	, SET_SHINY						= new TextureSet[] {addTextureSet(ModIDs.GT, F, "SHINY")         , addTextureSet(ModIDs.GT, T, "SHINY")         }
	, SET_SHARDS					= new TextureSet[] {addTextureSet(ModIDs.GT, F, "SHARDS")        , addTextureSet(ModIDs.GT, T, "SHARDS")        }
	, SET_POWDER					= new TextureSet[] {addTextureSet(ModIDs.GT, F, "POWDER")        , addTextureSet(ModIDs.GT, T, "POWDER")        }
	, SET_QUARTZ					= new TextureSet[] {addTextureSet(ModIDs.GT, F, "QUARTZ")        , addTextureSet(ModIDs.GT, T, "QUARTZ")        }
	, SET_EMERALD					= new TextureSet[] {addTextureSet(ModIDs.GT, F, "EMERALD")       , addTextureSet(ModIDs.GT, T, "EMERALD")       }
	, SET_DIAMOND					= new TextureSet[] {addTextureSet(ModIDs.GT, F, "DIAMOND")       , addTextureSet(ModIDs.GT, T, "DIAMOND")       }
	, SET_LIGNITE					= new TextureSet[] {addTextureSet(ModIDs.GT, F, "LIGNITE")       , addTextureSet(ModIDs.GT, T, "LIGNITE")       }
	, SET_MAGNETIC					= new TextureSet[] {addTextureSet(ModIDs.GT, F, "MAGNETIC")      , addTextureSet(ModIDs.GT, T, "MAGNETIC")      }
	, SET_METALLIC					= new TextureSet[] {addTextureSet(ModIDs.GT, F, "METALLIC")      , addTextureSet(ModIDs.GT, T, "METALLIC")      }
	, SET_CUBE_SHINY				= new TextureSet[] {addTextureSet(ModIDs.GT, F, "CUBE_SHINY")    , addTextureSet(ModIDs.GT, T, "CUBE_SHINY")    }
	, SET_NETHERSTAR				= new TextureSet[] {addTextureSet(ModIDs.GT, F, "NETHERSTAR")    , addTextureSet(ModIDs.GT, T, "NETHERSTAR")    }
	, SET_GEM_VERTICAL				= new TextureSet[] {addTextureSet(ModIDs.GT, F, "GEM_VERTICAL")  , addTextureSet(ModIDs.GT, T, "GEM_VERTICAL")  }
	, SET_GEM_HORIZONTAL			= new TextureSet[] {addTextureSet(ModIDs.GT, F, "GEM_HORIZONTAL"), addTextureSet(ModIDs.GT, T, "GEM_HORIZONTAL")}
	;
}