package gregapi.recipes;

import static gregapi.data.CS.*;
import static gregapi.data.OP.oreNormal;
import static gregapi.data.OP.orePoor;
import static gregapi.data.OP.oreRich;
import static gregapi.data.OP.oreSmall;
import gregapi.GT_API;
import gregapi.code.ArrayListNoNulls;
import gregapi.code.HashSetNoNulls;
import gregapi.code.ItemStackContainer;
import gregapi.code.ItemStackMap;
import gregapi.config.Config;
import gregapi.config.ConfigCategories;
import gregapi.data.CS.ConfigsGT;
import gregapi.data.CS.DirectoriesGT;
import gregapi.data.CS.ItemsGT;
import gregapi.data.*;
import gregapi.oredict.OreDictItemData;
import gregapi.oredict.OreDictManager;
import gregapi.oredict.OreDictMaterial;
import gregapi.oredict.OreDictMaterialStack;
import gregapi.random.IHasWorldAndCoords;
import gregapi.tileentity.base.TileEntityBase01Root;
import gregapi.util.OM;
import gregapi.util.UT;
import gregapi.util.UT.Fluids;

import java.util.*;

import net.minecraft.block.Block;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraftforge.fluids.Fluid;
import net.minecraftforge.fluids.FluidStack;
import net.minecraftforge.fluids.IFluidContainerItem;

/**
 * @author Gregorius Techneticies
 */
public class Recipe {
	public static class RecipeMap implements Runnable {
		/** RecipeMap-HashMap so that Machines can store their corresponding Recipe Lists as String NBT. */
		public static final Map<String, RecipeMap> RECIPE_MAPS = new HashMap<String, Recipe.RecipeMap>();
		
		public static final RecipeMap		sOreWasherRecipes							= new RecipeMapOreWasher			(null, "ic.recipe.orewasher"					, "Ore Washer"						, "ic2.blockOreWashingPlant"	, 0, 1, RES_PATH_GUI+"machines/OreWasher"						,/*IN-OUT-MIN-ITEM=*/ 1, 3, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 1,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F, F, T);
		public static final RecipeMap		sThermalCentrifugeRecipes					= new RecipeMapThermalCentrifuge	(null, "ic.recipe.thermalcentrifuge"			, "Thermal Centrifuge"				, "ic2.blockCentrifuge"			, 0, 1, RES_PATH_GUI+"machines/ThermalCentrifuge"				,/*IN-OUT-MIN-ITEM=*/ 1, 3, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 2, ""					,    1, ""		, T, F, F, T);
		public static final RecipeMap		sMaceratorRecipes							= new RecipeMapMacerator			(null, "ic.recipe.macerator"					, "Macerator"						, "ic2.macerator"				, 0, 1, RES_PATH_GUI+"machines/Macerator"						,/*IN-OUT-MIN-ITEM=*/ 1, 4, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F, F, T);
		public static final RecipeMap		sIC2CompressorRecipes						= new RecipeMapCompressor			(null, "ic.recipe.compressor"					, "Compressor"						, "ic2.compressor"				, 0, 1, RES_PATH_GUI+"machines/Compressor"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F, F, T);
		public static final RecipeMap		sExtractorRecipes							= new RecipeMapExtractor			(null, "ic.recipe.extractor"					, "Extractor"						, "ic2.extractor"				, 0, 1, RES_PATH_GUI+"machines/Extractor"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F, F, T);
		public static final RecipeMap		sRecyclerRecipes							= new RecipeMapRecycler				(null, "ic.recipe.recycler"						, "Recycler"						, "ic2.recycler"				, 0, 1, RES_PATH_GUI+"machines/Recycler"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F, F, T);
		public static final RecipeMap		sFurnaceRecipes								= new RecipeMapFurnace				(null, "mc.recipe.furnace"						, "Furnace"							, "smelting"					, 0, 1, RES_PATH_GUI+"machines/E_Furnace"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F, F, T);
		public static final RecipeMap		sMicrowaveRecipes							= new RecipeMapMicrowave			(null, "gt.recipe.microwave"					, "Microwave"						, "smelting"					, 0, 1, RES_PATH_GUI+"machines/E_Furnace"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F, F, T);
		public static final RecipeMap		sFurnaceFuel								= new RecipeMapFurnaceFuel			(null, "mc.recipe.furnacefuel"					, "Furnace Fuels"					, "smelting"					, 0, 1, RES_PATH_GUI+"machines/E_Furnace"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F, F, F);
		
		public static final RecipeMap		sScannerFakeRecipes							= new RecipeMap						(null, "gt.recipe.scanner"						, "Scanner"							, null							, 0, 1, RES_PATH_GUI+"machines/Scanner"							,/*IN-OUT-MIN-ITEM=*/ 2, 2, 1,/*IN-OUT-MIN-FLUID=*/ 1, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F/*T*/, F/*T*/, T);
		public static final RecipeMap		sRockBreakerFakeRecipes						= new RecipeMap						(null, "gt.recipe.rockbreaker"					, "Rock Breaker"					, null							, 0, 1, RES_PATH_GUI+"machines/RockBreaker"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 0,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F/*T*/, F/*T*/, T);
		public static final RecipeMap		sByProductList								= new RecipeMap						(null, "gt.recipe.byproductlist"				, "Ore Byproduct List"				, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 6, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, F, T);
		public static final RecipeMap		sCrucibleSmelting							= new RecipeMap						(null, "gt.recipe.cruciblesmelting"				, "Crucible Smelting"				, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 6, 6, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, "Temperature: "		,    1, " K"	, T, T, F, T);
		public static final RecipeMap		sCrucibleAlloying							= new RecipeMap						(null, "gt.recipe.cruciblealloying"				, "Combination Smelting"			, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 6, 6, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, "Temperature: "		,    1, " K"	, T, T, F, T);
		public static final RecipeMap		sGenerifierRecipes							= new RecipeMap						(null, "gt.recipe.generifier"					, "Generifier"						, null							, 0, 1, RES_PATH_GUI+"machines/Generifier"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 0,/*IN-OUT-MIN-FLUID=*/ 1, 1, 0,/*MIN*/ 1,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		
		public static final RecipeMap		sSharpeningRecipes							= new RecipeMap						(null, "gt.recipe.sharpener"					, "Sharpener"						, null							, 0, 1, RES_PATH_GUI+"machines/Sharpener"						,/*IN-OUT-MIN-ITEM=*/ 1, 2, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sSifterRecipes								= new RecipeMap						(null, "gt.recipe.sifter"						, "Sifter"							, null							, 2, 1, RES_PATH_GUI+"machines/Sifter"							,/*IN-OUT-MIN-ITEM=*/ 1,12, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sHammerRecipes								= new RecipeMapHammer				(null, "gt.recipe.hammer"						, "Hammer"							, null							, 6, 3, RES_PATH_GUI+"machines/Hammer"							,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F, T, T);
		public static final RecipeMap		sChiselRecipes								= new RecipeMapChisel				(null, "gt.recipe.chisel"						, "Chisel"							, null							, 0, 1, RES_PATH_GUI+"machines/Chisel"							,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F, T, T);
		public static final RecipeMap		sShredderRecipes							= new RecipeMapShredder				(null, "gt.recipe.shredder"						, "Shredder"						, null							, 0, 1, RES_PATH_GUI+"machines/Shredder"						,/*IN-OUT-MIN-ITEM=*/ 1, 6, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sCrusherRecipes								= new RecipeMapCrusher				(null, "gt.recipe.crusher"						, "Crusher"							, null							, 0, 1, RES_PATH_GUI+"machines/Crusher"							,/*IN-OUT-MIN-ITEM=*/ 1, 4, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sLatheRecipes								= new RecipeMap						(null, "gt.recipe.lathe"						, "Lathe"							, null							, 0, 1, RES_PATH_GUI+"machines/Lathe"							,/*IN-OUT-MIN-ITEM=*/ 1, 2, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sCutterRecipes								= new RecipeMapCutter				(null, "gt.recipe.cutter"						, "Cutter"							, null							, 0, 1, RES_PATH_GUI+"machines/Cutter"							,/*IN-OUT-MIN-ITEM=*/ 1, 2, 1,/*IN-OUT-MIN-FLUID=*/ 1, 0, 1,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sCoagulatorRecipes							= new RecipeMap						(null, "gt.recipe.coagulator"					, "Coagulator"						, null							, 0, 1, RES_PATH_GUI+"machines/Coagulator"						,/*IN-OUT-MIN-ITEM=*/ 0, 1, 0,/*IN-OUT-MIN-FLUID=*/ 1, 0, 1,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sSqueezerRecipes							= new RecipeMap						(null, "gt.recipe.squeezer"						, "Squeezer"						, null							, 0, 1, RES_PATH_GUI+"machines/Squeezer"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 1, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sJuicerRecipes								= new RecipeMap						(null, "gt.recipe.juicer"						, "Juicer"							, null							, 0, 1, RES_PATH_GUI+"machines/Juicer"							,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 1, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sCompressorRecipes							= new RecipeMap						(null, "gt.recipe.compressor"					, "Compressor"						, null							, 0, 1, RES_PATH_GUI+"machines/Compressor"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sCentrifugeRecipes							= new RecipeMap						(null, "gt.recipe.centrifuge"					, "Centrifuge"						, null							, 0, 1, RES_PATH_GUI+"machines/Centrifuge"						,/*IN-OUT-MIN-ITEM=*/ 1, 6, 0,/*IN-OUT-MIN-FLUID=*/ 1, 6, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sElectrolyzerRecipes						= new RecipeMap						(null, "gt.recipe.electrolyzer"					, "Electrolyzer"					, null							, 0, 1, RES_PATH_GUI+"machines/Electrolyzer"					,/*IN-OUT-MIN-ITEM=*/ 1, 6, 0,/*IN-OUT-MIN-FLUID=*/ 1, 6, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sRollingMillRecipes							= new RecipeMap						(null, "gt.recipe.rollingmill"					, "Rolling Mill"					, null							, 0, 1, RES_PATH_GUI+"machines/RollingMill"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sRollBenderRecipes							= new RecipeMap						(null, "gt.recipe.rollbender"					, "Roll Bender"						, null							, 0, 1, RES_PATH_GUI+"machines/RollBender"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sRollFormerRecipes							= new RecipeMap						(null, "gt.recipe.rollformer"					, "Roll Former"						, null							, 0, 1, RES_PATH_GUI+"machines/RollFormer"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sClusterMillRecipes							= new RecipeMap						(null, "gt.recipe.clustermill"					, "Cluster Mill"					, null							, 0, 1, RES_PATH_GUI+"machines/ClusterMill"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sWiremillRecipes							= new RecipeMap						(null, "gt.recipe.wiremill"						, "Wiremill"						, null							, 0, 1, RES_PATH_GUI+"machines/Wiremill"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sMixerRecipes								= new RecipeMap						(null, "gt.recipe.mixer"						, "Mixer"							, null							, 0, 1, RES_PATH_GUI+"machines/Mixer"							,/*IN-OUT-MIN-ITEM=*/ 6, 1, 0,/*IN-OUT-MIN-FLUID=*/ 6, 2, 0,/*MIN*/ 2,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sCannerRecipes								= new RecipeMapFluidCanner			(null, "gt.recipe.canner"						, "Canning Machine"					, null							, 0, 1, RES_PATH_GUI+"machines/Canner"							,/*IN-OUT-MIN-ITEM=*/ 2, 2, 1,/*IN-OUT-MIN-FLUID=*/ 1, 1, 0,/*MIN*/ 1,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sInjectorRecipes							= new RecipeMap						(null, "gt.recipe.injector"						, "Injector"						, null							, 0, 1, RES_PATH_GUI+"machines/Injector"						,/*IN-OUT-MIN-ITEM=*/ 2, 1, 0,/*IN-OUT-MIN-FLUID=*/ 2, 1, 0,/*MIN*/ 2,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sRoastingRecipes							= new RecipeMap						(null, "gt.recipe.roaster"						, "Roaster"							, null							, 0, 1, RES_PATH_GUI+"machines/Roaster"							,/*IN-OUT-MIN-ITEM=*/ 1, 3, 1,/*IN-OUT-MIN-FLUID=*/ 1, 1, 1,/*MIN*/ 2,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sDistilleryRecipes							= new RecipeMap						(null, "gt.recipe.distillery"					, "Distillery"						, null							, 0, 1, RES_PATH_GUI+"machines/Distillery"						,/*IN-OUT-MIN-ITEM=*/ 1, 2, 1,/*IN-OUT-MIN-FLUID=*/ 1, 2, 1,/*MIN*/ 2,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sExtruderRecipes							= new RecipeMap						(null, "gt.recipe.extruder"						, "Extruder"						, null							, 0, 1, RES_PATH_GUI+"machines/Extruder"						,/*IN-OUT-MIN-ITEM=*/ 2, 1, 2,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sFermenterRecipes							= new RecipeMap						(null, "gt.recipe.fermenter"					, "Fermenter"						, null							, 0, 1, RES_PATH_GUI+"machines/Fermenter"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 0,/*IN-OUT-MIN-FLUID=*/ 1, 1, 0,/*MIN*/ 1,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sPolarizerRecipes							= new RecipeMap						(null, "gt.recipe.polarizer"					, "Polarizer"						, null							, 0, 1, RES_PATH_GUI+"machines/Polarizer"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sLoomRecipes								= new RecipeMap						(null, "gt.recipe.loom"							, "Loom"							, null							, 0, 1, RES_PATH_GUI+"machines/Loom"							,/*IN-OUT-MIN-ITEM=*/ 6, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sCookingRecipes								= new RecipeMap						(null, "gt.recipe.cooker"						, "Cooker"							, null							, 0, 1, RES_PATH_GUI+"machines/Cooker"							,/*IN-OUT-MIN-ITEM=*/ 9, 1, 1,/*IN-OUT-MIN-FLUID=*/ 3, 1, 1,/*MIN*/ 2,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sPressRecipes								= new RecipeMapFormingPress			(null, "gt.recipe.press"						, "Press"							, null							, 0, 1, RES_PATH_GUI+"machines/Press"							,/*IN-OUT-MIN-ITEM=*/ 3, 1, 2,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sBathRecipes								= new RecipeMap						(null, "gt.recipe.bath"							, "Bath"							, null							, 0, 1, RES_PATH_GUI+"machines/Bath"							,/*IN-OUT-MIN-ITEM=*/ 6, 6, 1,/*IN-OUT-MIN-FLUID=*/ 1, 1, 1,/*MIN*/ 2,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sSmelterRecipes								= new RecipeMap						(null, "gt.recipe.smelter"						, "Smelter"							, null							, 0, 1, RES_PATH_GUI+"machines/Smelter"							,/*IN-OUT-MIN-ITEM=*/ 1, 1, 0,/*IN-OUT-MIN-FLUID=*/ 1, 1, 0,/*MIN*/ 1,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sLaserEngraverRecipes						= new RecipeMap						(null, "gt.recipe.laserengraver"				, "Precision Laser Engraver"		, null							, 0, 1, RES_PATH_GUI+"machines/LaserEngraver"					,/*IN-OUT-MIN-ITEM=*/ 2, 1, 2,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 2,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sCrystallisationCrucibleRecipes				= new RecipeMap						(null, "gt.recipe.crystallisationcrucible"		, "Crystallisation Crucible"		, null							, 0, 1, RES_PATH_GUI+"machines/CrystallisationCrucible"			,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 2, 0, 1,/*MIN*/ 1,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sScannerVisualsRecipes						= new RecipeMapScannerVisuals		(null, "gt.recipe.scannervisuals"				, "Scanner (Visuals)"				, null							, 0, 1, RES_PATH_GUI+"machines/ScannerVisuals"					,/*IN-OUT-MIN-ITEM=*/ 2, 2, 2,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 2,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		public static final RecipeMap		sPrinterRecipes								= new RecipeMapPrinter				(null, "gt.recipe.printer"						, "Printer"							, null							, 0, 1, RES_PATH_GUI+"machines/Printer"							,/*IN-OUT-MIN-ITEM=*/ 2, 1, 1,/*IN-OUT-MIN-FLUID=*/ 6, 0, 1,/*MIN*/ 2,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, T);
		
		public static final RecipeMap		sPlasmaArcFurnaceRecipes					= new RecipeMap						(null, "gt.recipe.plasmaarcfurnace"				, "Plasma Arc Furnace"				, null							, 0, 1, RES_PATH_GUI+"machines/PlasmaArcFurnace"				,/*IN-OUT-MIN-ITEM=*/ 1, 4, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 1,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F/*T*/, F/*T*/, T);
		public static final RecipeMap		sArcFurnaceRecipes							= new RecipeMap						(null, "gt.recipe.arcfurnace"					, "Arc Furnace"						, null							, 0, 1, RES_PATH_GUI+"machines/ArcFurnace"						,/*IN-OUT-MIN-ITEM=*/ 1, 4, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 1,/*MIN*/ 0,/*AMP=*/ 3, ""					,    1, ""		, T, F/*T*/, F/*T*/, T);
		public static final RecipeMap		sAutoclaveRecipes							= new RecipeMap						(null, "gt.recipe.autoclave"					, "Autoclave"						, null							, 0, 1, RES_PATH_GUI+"machines/Autoclave"						,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 1,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F/*T*/, F/*T*/, T);
		public static final RecipeMap		sElectroMagneticSeparatorRecipes			= new RecipeMap						(null, "gt.recipe.electromagneticseparator"		, "Electromagnetic Separator"		, null							, 0, 1, RES_PATH_GUI+"machines/ElectromagneticSeparator"		,/*IN-OUT-MIN-ITEM=*/ 1, 3, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F/*T*/, F/*T*/, T);
		public static final RecipeMap		sChemicalBathRecipes						= new RecipeMap						(null, "gt.recipe.chemicalbath"					, "Chemical Bath"					, null							, 0, 1, RES_PATH_GUI+"machines/ChemicalBath"					,/*IN-OUT-MIN-ITEM=*/ 1, 3, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 1,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F/*T*/, F/*T*/, T);
		public static final RecipeMap		sBrewingRecipes								= new RecipeMap						(null, "gt.recipe.brewer"						, "Brewing Machine"					, null							, 0, 1, RES_PATH_GUI+"machines/PotionBrewer"					,/*IN-OUT-MIN-ITEM=*/ 1, 0, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 1,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F/*T*/, F/*T*/, T);
		public static final RecipeMap		sFluidHeaterRecipes							= new RecipeMap						(null, "gt.recipe.fluidheater"					, "Fluid Heater"					, null							, 0, 1, RES_PATH_GUI+"machines/FluidHeater"						,/*IN-OUT-MIN-ITEM=*/ 1, 0, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 1,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F/*T*/, F/*T*/, T);
		public static final RecipeMap		sFluidSolidficationRecipes					= new RecipeMap						(null, "gt.recipe.fluidsolidifier"				, "Fluid Solidifier"				, null							, 0, 1, RES_PATH_GUI+"machines/FluidSolidifier"					,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 1,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F/*T*/, F/*T*/, T);
		public static final RecipeMap		sFluidExtractionRecipes						= new RecipeMap						(null, "gt.recipe.fluidextractor"				, "Fluid Extractor"					, null							, 0, 1, RES_PATH_GUI+"machines/FluidExtractor"					,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 1, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F/*T*/, F/*T*/, T);
		public static final RecipeMap		sBoxinatorRecipes							= new RecipeMap						(null, "gt.recipe.packager"						, "Packager"						, null							, 0, 1, RES_PATH_GUI+"machines/Packager"						,/*IN-OUT-MIN-ITEM=*/ 2, 1, 2,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F/*T*/, F/*T*/, T);
		public static final RecipeMap		sUnboxinatorRecipes							= new RecipeMapUnboxinator			(null, "gt.recipe.unpackager"					, "Unpackager"						, null							, 0, 1, RES_PATH_GUI+"machines/Unpackager"						,/*IN-OUT-MIN-ITEM=*/ 1, 2, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F/*T*/, F/*T*/, T);
		public static final RecipeMap		sFusionRecipes								= new RecipeMap						(null, "gt.recipe.fusionreactor"				, "Fusion Reactor"					, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 2, 1, 0,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, "Start: "				,    1, " EU"	, T, F/*F*/, F/*T*/, T);
		public static final RecipeMap		sBlastRecipes								= new RecipeMap						(null, "gt.recipe.blastfurnace"					, "Blast Furnace"					, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 2, 2, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, "Heat Capacity: "		,    1, " K"	, F, F/*T*/, F/*T*/, T);
		public static final RecipeMap		sImplosionRecipes							= new RecipeMap						(null, "gt.recipe.implosioncompressor"			, "Implosion Compressor"			, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 2, 2, 2,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F/*T*/, F/*T*/, T);
		public static final RecipeMap		sVacuumRecipes								= new RecipeMap						(null, "gt.recipe.vacuumfreezer"				, "Vacuum Freezer"					, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 1, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F/*T*/, F/*T*/, T);
		public static final RecipeMap		sChemicalRecipes							= new RecipeMap						(null, "gt.recipe.chemicalreactor"				, "Chemical Reactor"				, null							, 0, 1, RES_PATH_GUI+"machines/ChemicalReactor"					,/*IN-OUT-MIN-ITEM=*/ 2, 1, 1,/*IN-OUT-MIN-FLUID=*/ 1, 1, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F/*T*/, F/*T*/, T);
		public static final RecipeMap		sAlloySmelterRecipes						= new RecipeMap						(null, "gt.recipe.alloysmelter"					, "Alloy Smelter"					, null							, 0, 1, RES_PATH_GUI+"machines/AlloySmelter"					,/*IN-OUT-MIN-ITEM=*/ 2, 1, 2,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F/*T*/, F/*T*/, T);
		public static final RecipeMap		sAssemblerRecipes							= new RecipeMapAssembler			(null, "gt.recipe.assembler"					, "Assembler"						, null							, 0, 1, RES_PATH_GUI+"machines/Assembler"						,/*IN-OUT-MIN-ITEM=*/ 2, 1, 1,/*IN-OUT-MIN-FLUID=*/ 1, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F/*T*/, F/*T*/, T);
		public static final RecipeMap		sCNCRecipes									= new RecipeMap						(null, "gt.recipe.cncmachine"					, "CNC Machine"						, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 2, 1, 2,/*IN-OUT-MIN-FLUID=*/ 1, 0, 1,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F/*T*/, F/*T*/, T);
		public static final RecipeMap		sSlicerRecipes								= new RecipeMap						(null, "gt.recipe.slicer"						, "Slicer"							, null							, 0, 1, RES_PATH_GUI+"machines/Slicer"							,/*IN-OUT-MIN-ITEM=*/ 2, 1, 2,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F/*T*/, F/*T*/, T);
		public static final RecipeMap		sAmplifiers									= new RecipeMap						(null, "gt.recipe.uuamplifier"					, "UU Amplifier"					, null							, 0, 1, RES_PATH_GUI+"machines/Amplifabricator"					,/*IN-OUT-MIN-ITEM=*/ 1, 0, 1,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, ""					,    1, ""		, T, F/*T*/, F/*T*/, T);
		
		public static final RecipeMap		sFuelsBurn									= new RecipeMapFuel					(null, "gt.recipe.fuels.burn"					, "Burnable Fuels"					, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 2, 0,/*IN-OUT-MIN-FLUID=*/ 1, 2, 0,/*MIN*/ 1,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, F);
		public static final RecipeMap		sFuelsGas									= new RecipeMapFuel					(null, "gt.recipe.fuels.gas"					, "Gas Fuels"						, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 2, 0,/*IN-OUT-MIN-FLUID=*/ 1, 2, 0,/*MIN*/ 1,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, F);
		public static final RecipeMap		sFuelsHot									= new RecipeMapFuel					(null, "gt.recipe.fuels.hot"					, "Hot Fuels"						, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 2, 0,/*IN-OUT-MIN-FLUID=*/ 1, 2, 0,/*MIN*/ 1,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, F);
		public static final RecipeMap		sFuelsPlasma								= new RecipeMapFuel					(null, "gt.recipe.fuels.plasma"					, "Plasma Fuels"					, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 2, 0,/*IN-OUT-MIN-FLUID=*/ 1, 2, 0,/*MIN*/ 1,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, F);
		public static final RecipeMap		sFuelsEngine								= new RecipeMapFuel					(null, "gt.recipe.fuels.engine"					, "Engine Fuels"					, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 2, 0,/*IN-OUT-MIN-FLUID=*/ 1, 2, 0,/*MIN*/ 1,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, F);
		public static final RecipeMap		sFuelsTurbine								= new RecipeMapFuel					(null, "gt.recipe.fuels.turbine"				, "Turbine Fuels"					, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 2, 0,/*IN-OUT-MIN-FLUID=*/ 1, 2, 0,/*MIN*/ 1,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, F);
		public static final RecipeMap		sFuelsMagic									= new RecipeMapFuel					(null, "gt.recipe.fuels.magic"					, "Magic Fuels"						, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 2, 0,/*IN-OUT-MIN-FLUID=*/ 1, 2, 0,/*MIN*/ 1,/*AMP=*/ 1, ""					,    1, ""		, T, T, T, F);
		
		// Old Fuel Maps
		public static final RecipeMapFuel	sDieselFuels								= new RecipeMapFuel					(null, "gt.recipe.dieselgeneratorfuel"			, "Diesel Generator Fuel"			, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 1, 0,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, "Fuel Value: "		, 1000, " EU"	, T, F/*T*/, F/*T*/, F);
		public static final RecipeMapFuel	sTurbineFuels								= new RecipeMapFuel					(null, "gt.recipe.gasturbinefuel"				, "Gas Turbine Fuel"				, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 1, 0,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, "Fuel Value: "		, 1000, " EU"	, T, F/*T*/, F/*T*/, F);
		public static final RecipeMapFuel	sHotFuels									= new RecipeMapFuel					(null, "gt.recipe.thermalgeneratorfuel"			, "Thermal Generator Fuel"			, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 1, 0,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, "Fuel Value: "		, 1000, " EU"	, T, F/*F*/, F/*T*/, F);
		public static final RecipeMapFuel	sDenseLiquidFuels							= new RecipeMapFuel					(null, "gt.recipe.semifluidboilerfuels"			, "Semifluid Boiler Fuels"			, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 1, 0,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, "Fuel Value: "		, 1000, " EU"	, T, F/*T*/, F/*T*/, F);
		public static final RecipeMapFuel	sPlasmaFuels								= new RecipeMapFuel					(null, "gt.recipe.plasmageneratorfuels"			, "Plasma generator Fuels"			, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 1, 0,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, "Fuel Value: "		, 1000, " EU"	, T, F/*F*/, F/*T*/, F);
		public static final RecipeMapFuel	sMagicFuels									= new RecipeMapFuel					(null, "gt.recipe.magicfuels"					, "Magic Fuels"						, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 1, 0,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, "Fuel Value: "		, 1000, " EU"	, T, F/*F*/, F/*T*/, F);
		public static final RecipeMapFuel	sSmallNaquadahReactorFuels					= new RecipeMapFuel					(null, "gt.recipe.smallnaquadahreactor"			, "Small Naquadah Reactor"			, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 1, 0,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, "Fuel Value: "		, 1000, " EU"	, T, F/*T*/, F/*T*/, F);
		public static final RecipeMapFuel	sLargeNaquadahReactorFuels					= new RecipeMapFuel					(null, "gt.recipe.largenaquadahreactor"			, "Large Naquadah Reactor"			, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 1, 0,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, "Fuel Value: "		, 1000, " EU"	, T, F/*T*/, F/*T*/, F);
		public static final RecipeMapFuel	sFluidNaquadahReactorFuels					= new RecipeMapFuel					(null, "gt.recipe.fluidnaquadahreactor"			, "Fluid Naquadah Reactor"			, null							, 0, 1, RES_PATH_GUI+"machines/Default"							,/*IN-OUT-MIN-ITEM=*/ 1, 1, 0,/*IN-OUT-MIN-FLUID=*/ 0, 0, 0,/*MIN*/ 0,/*AMP=*/ 1, "Fuel Value: "		, 1000, " EU"	, T, F/*T*/, F/*T*/, F);
		
		/** List of Machines that can perform/use the Recipes of this Map. */
		public final List<ItemStack> mRecipeMachineList = new ArrayListNoNulls<ItemStack>();
		/** HashMap of Recipes based on their Items */
		public final Map<ItemStackContainer, Collection<Recipe>> mRecipeItemMap = new ItemStackMap<ItemStackContainer, Collection<Recipe>>();
		/** HashMap of Recipes based on their Fluids */
		public final Map<String, Collection<Recipe>> mRecipeFluidMap = new HashMap<String, Collection<Recipe>>();
		/** The List of all Recipes */
		public final Collection<Recipe> mRecipeList;
		/** String used as an unlocalised Name. */
		public final String mNameInternal;
		/** String used as a localised Name in things that shouldn't be localised, like Config File Names. */
		public final String mNameLocal, mNameLocalUnderscored;
		/** String used in NEI for the Recipe Lists. If null it will use the unlocalised Name instead*/
		public final String mNameNEI;
		/** GUI used for NEI Display. Usually the GUI of the Machine itself */
		public final String mGUIPath;
		public final String mNEISpecialValuePre, mNEISpecialValuePost;
		public final byte mProgressBarDirection, mProgressBarAmount;
		public final int mInputItemsCount, mOutputItemsCount, mInputFluidCount, mOutputFluidCount, mMinimalInputItems, mMinimalInputFluids, mMinimalInputs;
		public final long mNEISpecialValueMultiplier, mPower;
		public final boolean mNEIAllowed, mShowVoltageAmperageInNEI, mNeedsOutputs;
		/** Used to determine Input Tank Size. Contains the size of the largest FluidStack Input. */
		public int mMaxFluidInputSize = 1000;
		/** The Config File corresponding to this Recipe Handler. Will be initialised by GT_API. */
		public Config mConfigFile = null;
		
		/**
		 * Initialises a new type of Recipe Handler.
		 * @param aRecipeList a List you specify as Recipe List. Usually just an ArrayList with a pre-initialised Size.
		 * @param aNameInternal the unlocalised Name of this Recipe Handler, used mainly for NEI.
		 * @param aNameLocal the displayed Name inside the NEI Recipe GUI and regular ToolTips.
		 * @param aNEIGUIPath the displayed GUI Texture, usually just a Machine GUI. Auto-Attaches ".png" if forgotten.
		 * @param aInputItemsCount the usual amount of Input Slots this Recipe Class has.
		 * @param aOutputItemsCount the usual amount of Output Slots this Recipe Class has.
		 * @param aNEISpecialValuePre the String in front of the Special Value in NEI.
		 * @param aNEISpecialValueMultiplier the Value the Special Value is getting Multiplied with before displaying
		 * @param aNEISpecialValuePost the String after the Special Value. Usually for a Unit or something.
		 * @param aNEIAllowed if NEI is allowed to display this Recipe Handler in general.
		 */
		public RecipeMap(Collection<Recipe> aRecipeList, String aNameInternal, String aNameLocal, String aNameNEI, long aProgressBarDirection, long aProgressBarAmount, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aInputFluidCount, long aOutputFluidCount, long aMinimalInputFluids, long aMinimalInputs, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed, boolean aConfigAllowed, boolean aNeedsOutputs) {
			mNEIAllowed = aNEIAllowed;
			mShowVoltageAmperageInNEI = aShowVoltageAmperageInNEI;
			mNeedsOutputs = aNeedsOutputs;
			mRecipeList = (aRecipeList == null ? new HashSetNoNulls<Recipe>() : aRecipeList);
			mNameInternal = aNameInternal;
			mNameLocal = aNameLocal;
			mNameLocalUnderscored = mNameLocal.replaceAll(" ", "_");
			mNameNEI = aNameNEI == null ? mNameInternal : aNameNEI;
			mGUIPath = aNEIGUIPath.endsWith(".png")?aNEIGUIPath:aNEIGUIPath + ".png";
			mNEISpecialValuePre = aNEISpecialValuePre;
			mNEISpecialValueMultiplier = aNEISpecialValueMultiplier;
			mNEISpecialValuePost = aNEISpecialValuePost;
			mPower = aPower;
			mMinimalInputItems = (int)aMinimalInputItems;
			mInputItemsCount = (int)Math.max(aInputItemsCount, mMinimalInputItems);
			mOutputItemsCount = (int)aOutputItemsCount;
			mMinimalInputFluids = (int)aMinimalInputFluids;
			mInputFluidCount = (int)Math.max(aInputFluidCount, mMinimalInputFluids);
			mOutputFluidCount = (int)aOutputFluidCount;
			mMinimalInputs = (int)aMinimalInputs;
			mProgressBarDirection = (byte)aProgressBarDirection;
			mProgressBarAmount = (byte)aProgressBarAmount;
			LH.add(mNameInternal, mNameLocal);
			if (RECIPE_MAPS.containsKey(mNameInternal)) throw new IllegalArgumentException("Recipe Map Name already exists: " + mNameInternal);
			RECIPE_MAPS.put(mNameInternal, this);
			if (aConfigAllowed) if (GAPI.mBeforePreInit != null) GAPI.mBeforePreInit.add(this); else run();
		}
		
		@Override public String toString() {return mNameInternal;}
		@Override public void run() {mConfigFile = new Config(DirectoriesGT.CONFIG_RECIPES, mNameLocalUnderscored+".cfg");}
		
		@Deprecated
		public RecipeMap(Collection<Recipe> aRecipeList, String aNameInternal, String aNameLocal, String aNameNEI, long aProgressBarDirection, long aProgressBarAmount, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aInputFluidCount, long aOutputFluidCount, long aMinimalInputFluids, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed, boolean aConfigAllowed, boolean aNeedsOutputs) {
			this(aRecipeList, aNameInternal, aNameLocal, aNameNEI, aProgressBarDirection, aProgressBarAmount, aNEIGUIPath, aInputItemsCount, aOutputItemsCount, aMinimalInputItems, aInputFluidCount, aOutputFluidCount, aMinimalInputFluids, 0, aPower, aNEISpecialValuePre, aNEISpecialValueMultiplier, aNEISpecialValuePost, aShowVoltageAmperageInNEI, aNEIAllowed, aConfigAllowed, aNeedsOutputs);
		}
		
		@Deprecated
		public RecipeMap(Collection<Recipe> aRecipeList, String aNameInternal, String aNameLocal, String aNameNEI, long aProgressBarDirection, long aProgressBarAmount, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aInputFluidCount, long aOutputFluidCount, long aMinimalInputFluids, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed, boolean aConfigAllowed) {
			this(aRecipeList, aNameInternal, aNameLocal, aNameNEI, aProgressBarDirection, aProgressBarAmount, aNEIGUIPath, aInputItemsCount, aOutputItemsCount, aMinimalInputItems, aInputFluidCount, aOutputFluidCount, aMinimalInputFluids, 0, aPower, aNEISpecialValuePre, aNEISpecialValueMultiplier, aNEISpecialValuePost, aShowVoltageAmperageInNEI, aNEIAllowed, aConfigAllowed, T);
		}
		
		@Deprecated
		public RecipeMap(Collection<Recipe> aRecipeList, String aNameInternal, String aNameLocal, String aNameNEI, long aProgressBarDirection, long aProgressBarAmount, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aInputFluidCount, long aOutputFluidCount, long aMinimalInputFluids, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed) {
			this(aRecipeList, aNameInternal, aNameLocal, aNameNEI, aProgressBarDirection, aProgressBarAmount, aNEIGUIPath, aInputItemsCount, aOutputItemsCount, aMinimalInputItems, aInputFluidCount, aOutputFluidCount, aMinimalInputFluids, 0, aPower, aNEISpecialValuePre, aNEISpecialValueMultiplier, aNEISpecialValuePost, aShowVoltageAmperageInNEI, aNEIAllowed, T, T);
		}
		
		@Deprecated
		public RecipeMap(Collection<Recipe> aRecipeList, String aNameInternal, String aNameLocal, String aNameNEI, long aProgressBarDirection, long aProgressBarAmount, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aMinimalInputFluids, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed) {
			this(aRecipeList, aNameInternal, aNameLocal, aNameNEI, aProgressBarDirection, aProgressBarAmount, aNEIGUIPath, aInputItemsCount, aOutputItemsCount, aMinimalInputItems, 0, 0, aMinimalInputFluids, 0, aPower, aNEISpecialValuePre, aNEISpecialValueMultiplier, aNEISpecialValuePost, aShowVoltageAmperageInNEI, aNEIAllowed, T, T);
		}
		
		@Deprecated
		public RecipeMap(Collection<Recipe> aRecipeList, String aNameInternal, String aNameLocal, String aNameNEI, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aMinimalInputFluids, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed) {
			this(aRecipeList, aNameInternal, aNameLocal, aNameNEI, 0, 1, aNEIGUIPath, aInputItemsCount, aOutputItemsCount, aMinimalInputItems, 0, 0, aMinimalInputFluids, 0, aPower, aNEISpecialValuePre, aNEISpecialValueMultiplier, aNEISpecialValuePost, aShowVoltageAmperageInNEI, aNEIAllowed, T, T);
		}
		
		public Recipe addRecipe(boolean aOptimize, ItemStack[] aInputs, ItemStack[] aOutputs, Object aSpecial, long[] aOutputChances, FluidStack[] aFluidInputs, FluidStack[] aFluidOutputs, long aDuration, long aEUt, long aSpecialValue) {
			return addRecipe(new Recipe(aOptimize, T, aInputs, aOutputs, aSpecial, aOutputChances, aFluidInputs, aFluidOutputs, aDuration, aEUt, aSpecialValue));
		}
		
		public Recipe addRecipe(Recipe aRecipe) {
			return addRecipe(aRecipe, T, F, F, T);
		}
		
		// The Number next to the addRecipe is for the amount of Item Inputs used. X = Array
		
		public Recipe addRecipe1(boolean aOptimize, long aEUt, long aDuration, long   aChance	, ItemStack aInput																					, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput}				, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0));}
		public Recipe addRecipe2(boolean aOptimize, long aEUt, long aDuration, long   aChance	, ItemStack aInput1, ItemStack aInput2																, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput1, aInput2}	, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0));}
		public Recipe addRecipeX(boolean aOptimize, long aEUt, long aDuration, long   aChance	, ItemStack[] aInputs																				, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, T, aInputs								, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0));}
		public Recipe addRecipe1(boolean aOptimize, long aEUt, long aDuration					, ItemStack aInput																					, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput}				, aOutputs					, NI, ZL_LONG				, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0));}
		public Recipe addRecipe2(boolean aOptimize, long aEUt, long aDuration					, ItemStack aInput1, ItemStack aInput2																, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput1, aInput2}	, aOutputs					, NI, ZL_LONG				, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0));}
		public Recipe addRecipeX(boolean aOptimize, long aEUt, long aDuration					, ItemStack[] aInputs																				, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, aInputs								, aOutputs					, NI, ZL_LONG				, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0));}
		public Recipe addRecipe1(boolean aOptimize, long aEUt, long aDuration, long[] aChances	, ItemStack aInput																					, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput}				, aOutputs					, NI, aChances				, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0));}
		public Recipe addRecipe2(boolean aOptimize, long aEUt, long aDuration, long[] aChances	, ItemStack aInput1, ItemStack aInput2																, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput1, aInput2}	, aOutputs					, NI, aChances				, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0));}
		public Recipe addRecipeX(boolean aOptimize, long aEUt, long aDuration, long[] aChances	, ItemStack[] aInputs																				, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, aInputs								, aOutputs					, NI, aChances				, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0));}
		public Recipe addRecipe0(boolean aOptimize, long aEUt, long aDuration, long   aChance												, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, T, ZL_ITEMSTACK							, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe1(boolean aOptimize, long aEUt, long aDuration, long   aChance	, ItemStack aInput							, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput}				, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe2(boolean aOptimize, long aEUt, long aDuration, long   aChance	, ItemStack aInput1, ItemStack aInput2		, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput1, aInput2}	, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipeX(boolean aOptimize, long aEUt, long aDuration, long   aChance	, ItemStack[] aInputs						, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, T, aInputs								, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe0(boolean aOptimize, long aEUt, long aDuration																, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, ZL_ITEMSTACK							, aOutputs					, NI, ZL_LONG				, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe1(boolean aOptimize, long aEUt, long aDuration					, ItemStack aInput							, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput}				, aOutputs					, NI, ZL_LONG				, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe2(boolean aOptimize, long aEUt, long aDuration					, ItemStack aInput1, ItemStack aInput2		, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput1, aInput2}	, aOutputs					, NI, ZL_LONG				, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipeX(boolean aOptimize, long aEUt, long aDuration					, ItemStack[] aInputs						, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, aInputs								, aOutputs					, NI, ZL_LONG				, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe0(boolean aOptimize, long aEUt, long aDuration, long[] aChances												, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, ZL_ITEMSTACK							, aOutputs					, NI, aChances				, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe1(boolean aOptimize, long aEUt, long aDuration, long[] aChances	, ItemStack aInput							, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput}				, aOutputs					, NI, aChances				, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe2(boolean aOptimize, long aEUt, long aDuration, long[] aChances	, ItemStack aInput1, ItemStack aInput2		, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput1, aInput2}	, aOutputs					, NI, aChances				, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipeX(boolean aOptimize, long aEUt, long aDuration, long[] aChances	, ItemStack[] aInputs						, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, aInputs								, aOutputs					, NI, aChances				, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe0(boolean aOptimize, long aEUt, long aDuration																, FluidStack   aFluidInput , FluidStack... aFluidOutputs						) {return addRecipe(new Recipe(aOptimize, T, ZL_ITEMSTACK							, ZL_ITEMSTACK				, NI, ZL_LONG				, new FluidStack[] {aFluidInput}	, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipe1(boolean aOptimize, long aEUt, long aDuration					, ItemStack aInput							, FluidStack   aFluidInput , FluidStack... aFluidOutputs						) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput}				, ZL_ITEMSTACK				, NI, ZL_LONG				, new FluidStack[] {aFluidInput}	, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipe2(boolean aOptimize, long aEUt, long aDuration					, ItemStack aInput1, ItemStack aInput2		, FluidStack   aFluidInput , FluidStack... aFluidOutputs						) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput1, aInput2}	, ZL_ITEMSTACK				, NI, ZL_LONG				, new FluidStack[] {aFluidInput}	, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipeX(boolean aOptimize, long aEUt, long aDuration					, ItemStack[] aInputs						, FluidStack   aFluidInput , FluidStack... aFluidOutputs						) {return addRecipe(new Recipe(aOptimize, T, aInputs								, ZL_ITEMSTACK				, NI, ZL_LONG				, new FluidStack[] {aFluidInput}	, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipe0(boolean aOptimize, long aEUt, long aDuration, long   aChance												, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, T, ZL_ITEMSTACK							, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe1(boolean aOptimize, long aEUt, long aDuration, long   aChance	, ItemStack aInput							, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput}				, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe2(boolean aOptimize, long aEUt, long aDuration, long   aChance	, ItemStack aInput1, ItemStack aInput2		, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput1, aInput2}	, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipeX(boolean aOptimize, long aEUt, long aDuration, long   aChance	, ItemStack[] aInputs						, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, T, aInputs								, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe0(boolean aOptimize, long aEUt, long aDuration																, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, ZL_ITEMSTACK							, aOutputs					, NI, ZL_LONG				, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe1(boolean aOptimize, long aEUt, long aDuration					, ItemStack aInput							, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput}				, aOutputs					, NI, ZL_LONG				, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe2(boolean aOptimize, long aEUt, long aDuration					, ItemStack aInput1, ItemStack aInput2		, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput1, aInput2}	, aOutputs					, NI, ZL_LONG				, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipeX(boolean aOptimize, long aEUt, long aDuration					, ItemStack[] aInputs						, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, aInputs								, aOutputs					, NI, ZL_LONG				, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe0(boolean aOptimize, long aEUt, long aDuration, long[] aChances												, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, ZL_ITEMSTACK							, aOutputs					, NI, aChances				, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe1(boolean aOptimize, long aEUt, long aDuration, long[] aChances	, ItemStack aInput							, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput}				, aOutputs					, NI, aChances				, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe2(boolean aOptimize, long aEUt, long aDuration, long[] aChances	, ItemStack aInput1, ItemStack aInput2		, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput1, aInput2}	, aOutputs					, NI, aChances				, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipeX(boolean aOptimize, long aEUt, long aDuration, long[] aChances	, ItemStack[] aInputs						, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, aInputs								, aOutputs					, NI, aChances				, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0));}
		public Recipe addRecipe0(boolean aOptimize, long aEUt, long aDuration																, FluidStack[] aFluidInputs, FluidStack... aFluidOutputs						) {return addRecipe(new Recipe(aOptimize, T, ZL_ITEMSTACK							, ZL_ITEMSTACK				, NI, ZL_LONG				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipe1(boolean aOptimize, long aEUt, long aDuration					, ItemStack aInput							, FluidStack[] aFluidInputs, FluidStack... aFluidOutputs						) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput}				, ZL_ITEMSTACK				, NI, ZL_LONG				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipe2(boolean aOptimize, long aEUt, long aDuration					, ItemStack aInput1, ItemStack aInput2		, FluidStack[] aFluidInputs, FluidStack... aFluidOutputs						) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput1, aInput2}	, ZL_ITEMSTACK				, NI, ZL_LONG				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipeX(boolean aOptimize, long aEUt, long aDuration					, ItemStack[] aInputs						, FluidStack[] aFluidInputs, FluidStack... aFluidOutputs						) {return addRecipe(new Recipe(aOptimize, T, aInputs								, ZL_ITEMSTACK				, NI, ZL_LONG				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipe0(boolean aOptimize, long aEUt, long aDuration, long   aChance												, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, T, ZL_ITEMSTACK							, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipe1(boolean aOptimize, long aEUt, long aDuration, long   aChance	, ItemStack aInput							, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput}				, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipe2(boolean aOptimize, long aEUt, long aDuration, long   aChance	, ItemStack aInput1, ItemStack aInput2		, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput1, aInput2}	, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipeX(boolean aOptimize, long aEUt, long aDuration, long   aChance	, ItemStack[] aInputs						, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, T, aInputs								, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipe0(boolean aOptimize, long aEUt, long aDuration																, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, ZL_ITEMSTACK							, aOutputs					, NI, ZL_LONG				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipe1(boolean aOptimize, long aEUt, long aDuration					, ItemStack aInput							, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput}				, aOutputs					, NI, ZL_LONG				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipe2(boolean aOptimize, long aEUt, long aDuration					, ItemStack aInput1, ItemStack aInput2		, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput1, aInput2}	, aOutputs					, NI, ZL_LONG				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipeX(boolean aOptimize, long aEUt, long aDuration					, ItemStack[] aInputs						, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, aInputs								, aOutputs					, NI, ZL_LONG				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipe0(boolean aOptimize, long aEUt, long aDuration, long[] aChances												, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, ZL_ITEMSTACK							, aOutputs					, NI, aChances				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipe1(boolean aOptimize, long aEUt, long aDuration, long[] aChances	, ItemStack aInput							, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput}				, aOutputs					, NI, aChances				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipe2(boolean aOptimize, long aEUt, long aDuration, long[] aChances	, ItemStack aInput1, ItemStack aInput2		, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, new ItemStack[] {aInput1, aInput2}	, aOutputs					, NI, aChances				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0));}
		public Recipe addRecipeX(boolean aOptimize, long aEUt, long aDuration, long[] aChances	, ItemStack[] aInputs						, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, T, aInputs								, aOutputs					, NI, aChances				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0));}
		
		public Recipe addRecipe1(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long   aChance	, ItemStack aInput																					, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput}				, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe2(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long   aChance	, ItemStack aInput1, ItemStack aInput2																, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput1, aInput2}	, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipeX(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long   aChance	, ItemStack[] aInputs																				, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, aInputs								, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe1(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack aInput																					, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput}				, aOutputs					, NI, ZL_LONG				, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe2(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack aInput1, ItemStack aInput2																, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput1, aInput2}	, aOutputs					, NI, ZL_LONG				, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipeX(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack[] aInputs																				, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, aInputs								, aOutputs					, NI, ZL_LONG				, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe1(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long[] aChances	, ItemStack aInput																					, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput}				, aOutputs					, NI, aChances				, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe2(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long[] aChances	, ItemStack aInput1, ItemStack aInput2																, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput1, aInput2}	, aOutputs					, NI, aChances				, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipeX(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long[] aChances	, ItemStack[] aInputs																				, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, aInputs								, aOutputs					, NI, aChances				, ZL_FLUIDSTACK						, ZL_FLUIDSTACK						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe0(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long   aChance												, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, ZL_ITEMSTACK							, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe1(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long   aChance	, ItemStack aInput							, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput}				, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe2(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long   aChance	, ItemStack aInput1, ItemStack aInput2		, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput1, aInput2}	, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipeX(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long   aChance	, ItemStack[] aInputs						, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, aInputs								, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe0(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration																, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, ZL_ITEMSTACK							, aOutputs					, NI, ZL_LONG				, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe1(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack aInput							, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput}				, aOutputs					, NI, ZL_LONG				, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe2(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack aInput1, ItemStack aInput2		, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput1, aInput2}	, aOutputs					, NI, ZL_LONG				, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipeX(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack[] aInputs						, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, aInputs								, aOutputs					, NI, ZL_LONG				, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe0(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long[] aChances												, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, ZL_ITEMSTACK							, aOutputs					, NI, aChances				, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe1(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long[] aChances	, ItemStack aInput							, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput}				, aOutputs					, NI, aChances				, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe2(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long[] aChances	, ItemStack aInput1, ItemStack aInput2		, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput1, aInput2}	, aOutputs					, NI, aChances				, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipeX(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long[] aChances	, ItemStack[] aInputs						, FluidStack   aFluidInput , FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, aInputs								, aOutputs					, NI, aChances				, new FluidStack[] {aFluidInput}	, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe0(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration																, FluidStack   aFluidInput , FluidStack... aFluidOutputs						) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, ZL_ITEMSTACK							, ZL_ITEMSTACK				, NI, ZL_LONG				, new FluidStack[] {aFluidInput}	, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe1(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack aInput							, FluidStack   aFluidInput , FluidStack... aFluidOutputs						) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput}				, ZL_ITEMSTACK				, NI, ZL_LONG				, new FluidStack[] {aFluidInput}	, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe2(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack aInput1, ItemStack aInput2		, FluidStack   aFluidInput , FluidStack... aFluidOutputs						) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput1, aInput2}	, ZL_ITEMSTACK				, NI, ZL_LONG				, new FluidStack[] {aFluidInput}	, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipeX(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack[] aInputs						, FluidStack   aFluidInput , FluidStack... aFluidOutputs						) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, aInputs								, ZL_ITEMSTACK				, NI, ZL_LONG				, new FluidStack[] {aFluidInput}	, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe0(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long   aChance												, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, ZL_ITEMSTACK							, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe1(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long   aChance	, ItemStack aInput							, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput}				, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe2(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long   aChance	, ItemStack aInput1, ItemStack aInput2		, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput1, aInput2}	, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipeX(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long   aChance	, ItemStack[] aInputs						, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, aInputs								, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe0(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration																, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, ZL_ITEMSTACK							, aOutputs					, NI, ZL_LONG				, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe1(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack aInput							, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput}				, aOutputs					, NI, ZL_LONG				, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe2(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack aInput1, ItemStack aInput2		, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput1, aInput2}	, aOutputs					, NI, ZL_LONG				, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipeX(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack[] aInputs						, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, aInputs								, aOutputs					, NI, ZL_LONG				, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe0(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long[] aChances												, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, ZL_ITEMSTACK							, aOutputs					, NI, aChances				, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe1(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long[] aChances	, ItemStack aInput							, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput}				, aOutputs					, NI, aChances				, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe2(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long[] aChances	, ItemStack aInput1, ItemStack aInput2		, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput1, aInput2}	, aOutputs					, NI, aChances				, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipeX(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long[] aChances	, ItemStack[] aInputs						, FluidStack[] aFluidInputs, FluidStack    aFluidOutput	, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, aInputs								, aOutputs					, NI, aChances				, aFluidInputs						, new FluidStack[] {aFluidOutput}	, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe0(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration																, FluidStack[] aFluidInputs, FluidStack... aFluidOutputs						) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, ZL_ITEMSTACK							, ZL_ITEMSTACK				, NI, ZL_LONG				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe1(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack aInput							, FluidStack[] aFluidInputs, FluidStack... aFluidOutputs						) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput}				, ZL_ITEMSTACK				, NI, ZL_LONG				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe2(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack aInput1, ItemStack aInput2		, FluidStack[] aFluidInputs, FluidStack... aFluidOutputs						) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput1, aInput2}	, ZL_ITEMSTACK				, NI, ZL_LONG				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipeX(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack[] aInputs						, FluidStack[] aFluidInputs, FluidStack... aFluidOutputs						) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, aInputs								, ZL_ITEMSTACK				, NI, ZL_LONG				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe0(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long   aChance												, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, ZL_ITEMSTACK							, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe1(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long   aChance	, ItemStack aInput							, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput}				, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe2(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long   aChance	, ItemStack aInput1, ItemStack aInput2		, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput1, aInput2}	, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipeX(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long   aChance	, ItemStack[] aInputs						, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack    aOutput	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, aInputs								, new ItemStack[] {aOutput}	, NI, new long[] {aChance}	, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe0(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration																, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, ZL_ITEMSTACK							, aOutputs					, NI, ZL_LONG				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe1(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack aInput							, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput}				, aOutputs					, NI, ZL_LONG				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe2(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack aInput1, ItemStack aInput2		, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput1, aInput2}	, aOutputs					, NI, ZL_LONG				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipeX(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration					, ItemStack[] aInputs						, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, aInputs								, aOutputs					, NI, ZL_LONG				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe0(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long[] aChances												, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, ZL_ITEMSTACK							, aOutputs					, NI, aChances				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe1(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long[] aChances	, ItemStack aInput							, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput}				, aOutputs					, NI, aChances				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipe2(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long[] aChances	, ItemStack aInput1, ItemStack aInput2		, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, new ItemStack[] {aInput1, aInput2}	, aOutputs					, NI, aChances				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		public Recipe addRecipeX(boolean aOptimize, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors, long aEUt, long aDuration, long[] aChances	, ItemStack[] aInputs						, FluidStack[] aFluidInputs, FluidStack[]  aFluidOutputs, ItemStack... aOutputs	) {return addRecipe(new Recipe(aOptimize, aCheckForCollisions, aInputs								, aOutputs					, NI, aChances				, aFluidInputs						, aFluidOutputs						, aDuration, aEUt, 0), aCheckForCollisions, aFakeRecipe, aHidden, aLogErrors);}
		
		protected Recipe addRecipe(Recipe aRecipe, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden, boolean aLogErrors) {
			aRecipe.mHidden = aHidden;
			aRecipe.mFakeRecipe = aFakeRecipe;
			if (aCheckForCollisions && findRecipe(null, null, F, Long.MAX_VALUE, null, aRecipe.mFluidInputs, aRecipe.mInputs) != null) return null;
			return add(aRecipe, aLogErrors);
		}
		
		protected Recipe addRecipe(Recipe aRecipe, boolean aCheckForCollisions, boolean aFakeRecipe, boolean aHidden) {
			return addRecipe(aRecipe, aCheckForCollisions, aFakeRecipe, aHidden, T);
		}
		
		/** Only used for fake Recipe Handlers to show something in NEI, do not use this for adding actual Recipes! findRecipe wont find fake Recipes, containsInput WILL find fake Recipes */
		public Recipe addFakeRecipe(boolean aCheckForCollisions, ItemStack[] aInputs, ItemStack[] aOutputs, Object aSpecial, long[] aOutputChances, FluidStack[] aFluidInputs, FluidStack[] aFluidOutputs, long aDuration, long aEUt, long aSpecialValue) {
			return addFakeRecipe(aCheckForCollisions, new Recipe(F, F, aInputs, aOutputs, aSpecial, aOutputChances, aFluidInputs, aFluidOutputs, aDuration, aEUt, aSpecialValue));
		}
		
		/** Only used for fake Recipe Handlers to show something in NEI, do not use this for adding actual Recipes! findRecipe wont find fake Recipes, containsInput WILL find fake Recipes */
		public Recipe addFakeRecipe(boolean aCheckForCollisions, ItemStack[] aInputs, ItemStack[] aOutputs, Object aSpecial, FluidStack[] aFluidInputs, FluidStack[] aFluidOutputs, long aDuration, long aEUt, long aSpecialValue) {
			return addFakeRecipe(aCheckForCollisions, new Recipe(F, F, aInputs, aOutputs, aSpecial, null, aFluidInputs, aFluidOutputs, aDuration, aEUt, aSpecialValue));
		}
		
		/** Only used for fake Recipe Handlers to show something in NEI, do not use this for adding actual Recipes! findRecipe wont find fake Recipes, containsInput WILL find fake Recipes */
		public Recipe addFakeRecipe(boolean aCheckForCollisions, Recipe aRecipe) {
			return addRecipe(aRecipe, aCheckForCollisions, T, F, T);
		}
		
		public Recipe add(Recipe aRecipe) {
			return add(aRecipe, T);
		}
		
		public Recipe add(Recipe aRecipe, boolean aLogErrors) {
			if (!aRecipe.mFakeRecipe) {
				boolean tErrored = F, tFailed = F;
				if (aRecipe.mInputs.length + aRecipe.mFluidInputs.length <= 0) {
					if (aLogErrors) ERR.println("Error: Recipe has no Inputs!");
					tFailed = T;
				} else {
					if (mNeedsOutputs && aRecipe.mOutputs.length + aRecipe.mFluidOutputs.length <= 0) {if (aLogErrors) ERR.println("Error: Recipe has no Outputs!"											); tFailed = T;}
					if (aRecipe.mInputs			.length < mMinimalInputItems						) {if (aLogErrors) ERR.println("Error: Recipe has less than the minimal amount of Input ItemStacks!"		); tFailed = T;}
					if (aRecipe.mFluidInputs	.length < mMinimalInputFluids						) {if (aLogErrors) ERR.println("Error: Recipe has less than the minimal amount of Input FluidStacks!"	); tFailed = T;}
					if (aRecipe.mFluidInputs.length + aRecipe.mInputs.length < mMinimalInputs		) {if (aLogErrors) ERR.println("Error: Recipe has less than the minimal amount of general Inputs!"		); tFailed = T;}
					if (aRecipe.mInputs			.length > mInputItemsCount							) {if (aLogErrors) ERR.println("Error: Recipe has more than the maximum amount of Input ItemStacks!"		); tFailed = T;}
					if (aRecipe.mFluidInputs	.length > mInputFluidCount							) {if (aLogErrors) ERR.println("Error: Recipe has more than the maximum amount of Input FluidStacks!"	); tFailed = T;}
				}
				if (aRecipe.mOutputs			.length > mOutputItemsCount							) {if (aLogErrors) ERR.println("WARNING: Recipe has more than the maximum amount of Output ItemStacks!"	); tErrored = T;}
				if (aRecipe.mFluidOutputs		.length > mOutputFluidCount							) {if (aLogErrors) ERR.println("WARNING: Recipe has more than the maximum amount of Output FluidStacks!"	); tErrored = T;}
				if (aRecipe.mDuration															== 0) {if (aLogErrors) ERR.println("WARNING: Recipe has no Duration Value!"									); tErrored = T;}
				
				if (tErrored || tFailed) {
					if (aLogErrors) {int i = 0; for (StackTraceElement tElement : new Exception().getStackTrace()) if (!tElement.getClassName().equals("gregapi.recipes.Recipe$RecipeMap")) if (i++<5 && !tElement.getClassName().startsWith("sun")) ERR.println("\tat " + tElement); else break;}
					if (tFailed) return null;
				}
			}
			
			if (mConfigFile != null) {
				String tConfigName = "";
				if (aRecipe.mInputs.length > 0) tConfigName += UT.Stacks.configNames(aRecipe.mInputs);
				if (aRecipe.mFluidInputs.length > 0) tConfigName += UT.Fluids.configNames(aRecipe.mFluidInputs);
				if (UT.Code.stringValid(tConfigName)) {
					aRecipe.mDuration = mConfigFile.get(mNameLocalUnderscored, tConfigName, aRecipe.mEnabled?aRecipe.mDuration:0);
					aRecipe.mEnabled = (aRecipe.mDuration > 0);
				}
			}
			
			if (!mRecipeList.add(aRecipe)) return null;
			for (FluidStack aFluid : aRecipe.mFluidInputs) if (aFluid != null) {
				mMaxFluidInputSize = Math.max(mMaxFluidInputSize, aFluid.amount);
				Collection<Recipe> tList = mRecipeFluidMap.get(aFluid.getFluid().getName());
				if (tList == null) mRecipeFluidMap.put(aFluid.getFluid().getName(), tList = new HashSet<Recipe>(1));
				tList.add(aRecipe);
			}
			return addToItemMap(aRecipe);
		}
		
		public void reInit() {
        	mRecipeItemMap.clear();
        	for (Recipe tRecipe : mRecipeList) {
        		OreDictManager.INSTANCE.setStackArray(T, tRecipe.mInputs);
        		OreDictManager.INSTANCE.setStackArray(T, tRecipe.mOutputs);
            	addToItemMap(tRecipe);
        	}
		}
		
		/** @return if this Item is a valid Input for any for the Recipes */
		public boolean containsInput(ItemStack aStack) {
			return aStack != null && (mRecipeItemMap.containsKey(new ItemStackContainer(aStack)) || mRecipeItemMap.containsKey(new ItemStackContainer(UT.Stacks.copyMeta(W, aStack))));
		}
		
		/** @return if this Fluid is a valid Input for any for the Recipes */
		public boolean containsInput(FluidStack aFluid) {
			return aFluid != null && containsInput(aFluid.getFluid());
		}
		
		/** @return if this Fluid is a valid Input for any for the Recipes */
		public boolean containsInput(Fluid aFluid) {
			return aFluid != null && mRecipeFluidMap.containsKey(aFluid.getName());
		}
		
		private Recipe oRecipe = null;
		
		/**
		 * finds a Recipe matching the aFluid and ItemStack Inputs.
		 * @param aTileEntity an Object representing the current coordinates of the executing Block/Entity/Whatever. This may be null, especially during Startup.
		 * @param aRecipe in case this is != null it will try to use this Recipe first when looking things up.
		 * @param aNotUnificated if this is T the Recipe searcher will unificate the ItemStack Inputs
		 * @param aSize Voltage of the Machine or Long.MAX_VALUE if it has no Voltage
		 * @param aFluids the Fluid Inputs
		 * @param aSpecialSlot the content of the Special Slot, the regular Manager doesn't do anything with this, but some custom ones do.
		 * @param aInputs the Item Inputs
		 * @return the Recipe it has found or null for no matching Recipe
		 */
		public Recipe findRecipe(IHasWorldAndCoords aTileEntity, Recipe aRecipe, boolean aNotUnificated, long aSize, ItemStack aSpecialSlot, FluidStack[] aFluids, ItemStack... aInputs) {
			// No Recipes? Well, nothing to be found then.
			if (mRecipeList.isEmpty()) return null;
			
			// Some Recipe Classes require a certain amount of Inputs of certain kinds. Like "at least 1 Fluid + 1 Stack" or "at least 2 Stacks" before they start searching for Recipes.
			// This improves Performance massively, especially if people leave things like Circuits, Molds or Shapes in their Machines to select Sub Recipes.
			if (GAPI_POST.mFinishedServerStarted > 0) {
				long tItemAmount = UT.Code.getNonNulls(aInputs);
				if (tItemAmount < mMinimalInputItems) return null;
				long tFluidAmount = UT.Code.getNonNulls(aFluids);
				if (tFluidAmount < mMinimalInputFluids) return null;
				if (tFluidAmount + tItemAmount < mMinimalInputs) return null;
			}
			
			// Unification happens here in case the Input isn't already unificated.
			if (aNotUnificated) aInputs = OreDictManager.INSTANCE.getStackArray(T, (Object[])aInputs);
			
			// Check the Recipe which has been used last time in order to not have to search for it again, if possible.
			if (aRecipe != null) if (!aRecipe.mFakeRecipe && aRecipe.mCanBeBuffered && aRecipe.isRecipeInputEqual(F, T, aFluids, aInputs)) return aRecipe.mEnabled&&UT.Code.abs_greater_equal(aSize*mPower, aRecipe.mEUt)?oRecipe=aRecipe:null;
			if (oRecipe != null) if (!oRecipe.mFakeRecipe && oRecipe.mCanBeBuffered && oRecipe.isRecipeInputEqual(F, T, aFluids, aInputs)) return oRecipe.mEnabled&&UT.Code.abs_greater_equal(aSize*mPower, oRecipe.mEUt)?oRecipe:null;
			
			// Now look for the Recipes inside the Item HashMaps, but only when the Recipes usually have Items.
			if (mInputItemsCount > 0 && aInputs != null) for (ItemStack tStack1 : aInputs) if (tStack1 != null) {
				Collection<Recipe>
				tRecipes = mRecipeItemMap.get(new ItemStackContainer(tStack1));
				if (tRecipes != null) for (Recipe tRecipe : tRecipes) if (!tRecipe.mFakeRecipe && tRecipe.isRecipeInputEqual(F, T, aFluids, aInputs)) return tRecipe.mEnabled&&UT.Code.abs_greater_equal(aSize*mPower, tRecipe.mEUt)?oRecipe=tRecipe:null;
				
				tRecipes = mRecipeItemMap.get(new ItemStackContainer(UT.Stacks.copyMeta(W, tStack1)));
				if (tRecipes != null) for (Recipe tRecipe : tRecipes) if (!tRecipe.mFakeRecipe && tRecipe.isRecipeInputEqual(F, T, aFluids, aInputs)) return tRecipe.mEnabled&&UT.Code.abs_greater_equal(aSize*mPower, tRecipe.mEUt)?oRecipe=tRecipe:null;
				
				ItemStack tStack2 = OreDictManager.INSTANCE.getStack_(F, tStack1);
				
				if (!UT.Stacks.equal(tStack1, tStack2, true)) {
				tRecipes = mRecipeItemMap.get(new ItemStackContainer(tStack2));
				if (tRecipes != null) for (Recipe tRecipe : tRecipes) if (!tRecipe.mFakeRecipe && tRecipe.isRecipeInputEqual(F, T, aFluids, aInputs)) return tRecipe.mEnabled&&UT.Code.abs_greater_equal(aSize*mPower, tRecipe.mEUt)?oRecipe=tRecipe:null;
				}
				if (tStack1.getItem() != tStack2.getItem()) {
				tRecipes = mRecipeItemMap.get(new ItemStackContainer(UT.Stacks.copyMeta(W, tStack2)));
				if (tRecipes != null) for (Recipe tRecipe : tRecipes) if (!tRecipe.mFakeRecipe && tRecipe.isRecipeInputEqual(F, T, aFluids, aInputs)) return tRecipe.mEnabled&&UT.Code.abs_greater_equal(aSize*mPower, tRecipe.mEUt)?oRecipe=tRecipe:null;
				}
			}
			
			// If the minimal Amount of Items for the Recipe is 0, then it could be a Fluid-Only Recipe, so check that Map too.
			if (mInputFluidCount > 0 && mMinimalInputItems == 0 && aFluids != null) for (FluidStack aFluid : aFluids) if (aFluid != null) {
				Collection<Recipe>
				tRecipes = mRecipeFluidMap.get(aFluid.getFluid().getName());
				if (tRecipes != null) for (Recipe tRecipe : tRecipes) if (!tRecipe.mFakeRecipe && tRecipe.isRecipeInputEqual(F, T, aFluids, aInputs)) return tRecipe.mEnabled&&UT.Code.abs_greater_equal(aSize*mPower, tRecipe.mEUt)?oRecipe=tRecipe:null;
			}
			
			// And nothing has been found.
			return null;
		}
		
		public List<Recipe> getNEIAllRecipes() {
			ArrayListNoNulls<Recipe> rList = new ArrayListNoNulls<Recipe>();
			for (Recipe tRecipe : mRecipeList) if (!tRecipe.mHidden) rList.add(tRecipe);
			return rList;
		}
		
		public List<Recipe> getNEIRecipes(ItemStack... aOutputs) {
			ArrayListNoNulls<Recipe> rList = new ArrayListNoNulls<Recipe>();
			for (Recipe tRecipe : mRecipeList) if (!tRecipe.mHidden) {
				for (ItemStack aOutput : aOutputs) if (aOutput != null) {
					if (IL.Display_Fluid.equal(aOutput, T, T)) {
						for (FluidStack tOutput : tRecipe.mFluidOutputs) {
							if (tOutput.getFluidID() == UT.Stacks.meta(aOutput)) {
								rList.add(tRecipe);
								break;
							}
						}
					} else {
						for (ItemStack tOutput : tRecipe.mOutputs) if (tOutput != null && tOutput.getItem() == aOutput.getItem()) {
							if (UT.Stacks.meta(tOutput) == W || UT.Stacks.meta(tOutput) == UT.Stacks.meta(aOutput) || tOutput.isItemStackDamageable()) {
								rList.add(tRecipe);
								break;
							}
						}
					}
				}
			}
			return rList;
		}
		
		public List<Recipe> getNEIUsages(ItemStack... aInputs) {
			ArrayListNoNulls<Recipe> rList = new ArrayListNoNulls<Recipe>();
			for (Recipe tRecipe : mRecipeList) if (!tRecipe.mHidden) {
				for (ItemStack aInput : aInputs) if (aInput != null) {
					if (IL.Display_Fluid.equal(aInput, T, T)) {
						for (FluidStack tInput : tRecipe.mFluidInputs) {
							if (tInput.getFluidID() == UT.Stacks.meta(aInput)) {
								rList.add(tRecipe);
								break;
							}
						}
					} else {
						for (ItemStack tInput : tRecipe.mInputs) if (tInput != null && tInput.getItem() == aInput.getItem()) {
							if (UT.Stacks.meta(tInput) == W || UT.Stacks.meta(tInput) == UT.Stacks.meta(aInput) || tInput.isItemStackDamageable()) {
								rList.add(tRecipe);
								break;
							}
						}
					}
				}
			}
			return rList;
		}
		
		protected Recipe addToItemMap(Recipe aRecipe) {
			for (ItemStack aStack : aRecipe.mInputs) if (aStack != null) {
				ItemStackContainer tStack = new ItemStackContainer(aStack);
				Collection<Recipe> tList = mRecipeItemMap.get(tStack);
				if (tList == null) mRecipeItemMap.put(tStack, tList = new HashSet<Recipe>(1));
				tList.add(aRecipe);
			}
			return aRecipe;
		}
		
		public boolean openNEI() {
			try {codechicken.nei.recipe.GuiCraftingRecipe.openRecipeGui(mNameNEI, new Object[0]); return true;} catch(Throwable e) {/**/}
			return false;
		}
	}
	
	public static void reInit() {
        for (RecipeMap tMapEntry : RecipeMap.RECIPE_MAPS.values()) tMapEntry.reInit();
	}
	
	/** If you want to change the Output, feel free to modify or even replace the whole ItemStack Array, for Inputs, please add a new Recipe, because of the HashMaps. */
	public ItemStack[] mInputs, mOutputs;
	/** If you want to change the Output, feel free to modify or even replace the whole ItemStack Array, for Inputs, please add a new Recipe, because of the HashMaps. */
	public FluidStack[] mFluidInputs, mFluidOutputs;
	/** If you changed the amount of Array-Items inside the Output Array then the length of this Array must be larger or equal to the Output Array. A chance of 10000 equals 100% if the MaxChance Value is default. */
	public long[] mChances, mMaxChances;
	/** An Item that needs to be inside the Special Slot, like for example the Copy Slot inside the Printer. This is only useful for Fake Recipes in NEI, since findRecipe() and containsInput() don't give a shit about this Field. Lists are also possible. */
	public Object mSpecialItems;
	
	public long mDuration, mEUt, mSpecialValue;
	
	/** Use this to just disable a specific Recipe, but the Configuration enables that already for every single Recipe. */
	public boolean mEnabled = T;
	/** If this Recipe is hidden from NEI */
	public boolean mHidden = F;
	/** If this Recipe is Fake and therefore doesn't get found by the findRecipe Function (It is still in the HashMaps, so that containsInput does return T on those fake Inputs) */
	public boolean mFakeRecipe = F;
	/** If this Recipe can be stored inside a Machine in order to make Recipe searching more Efficient by trying the previously used Recipe first. In case you have a Recipe Map overriding things and returning one time use Recipes, you have to set this to F. */
	public boolean mCanBeBuffered = T;
	/** If this Recipe needs the Output Slots to be completely empty. Needed in case you have randomised Outputs */
	public boolean mNeedsEmptyOutput = F;
	
	public int getOutputChance(long aIndex) {if (aIndex < 0 || aIndex >= mChances.length) return 10000; return (int)mChances[(int)aIndex];}
	public int getMaxChance(long aIndex) {if (aIndex < 0 || aIndex >= mMaxChances.length) return 10000; return (int)mMaxChances[(int)aIndex];}
	
	public ItemStack getRepresentativeInput(long aIndex) {if (aIndex < 0 || aIndex >= mInputs.length) return null; return UT.Stacks.copy(mInputs[(int)aIndex]);}
	public ItemStack getOutput(long aIndex) {if (aIndex < 0 || aIndex >= mOutputs.length) return null; return UT.Stacks.copy(mOutputs[(int)aIndex]);}
	
	public FluidStack getRepresentativeFluidInput(long aIndex) {if (aIndex < 0 || aIndex >= mFluidInputs.length || mFluidInputs[(int)aIndex] == null) return null; return mFluidInputs[(int)aIndex].copy();}
	public FluidStack getFluidOutput(long aIndex) {if (aIndex < 0 || aIndex >= mFluidOutputs.length || mFluidOutputs[(int)aIndex] == null) return null; return mFluidOutputs[(int)aIndex].copy();}
	
	public Recipe copy() {
		return new Recipe(this);
	}
	
	public FluidStack[] getFluidOutputs(Random aRandom) {
		FluidStack[] rArray = new FluidStack[mFluidOutputs.length];
		for (int i = 0; i < rArray.length; i++) rArray[i] = getFluidOutput(i);
		return rArray;
	}
	
	public ItemStack[] getOutputs(Random aRandom) {
		ItemStack[] rArray = new ItemStack[mOutputs.length];
		if (aRandom == null) aRandom = new Random();
		for (int i = 0; i < rArray.length; i++) {
			ItemStack tOutput = getOutput(i);
			if (tOutput != null) {
				int tChance = getOutputChance(i);
				if (tChance > 0) {
					int tMax = getMaxChance(i);
					if (tChance == tMax) {
						rArray[i] = tOutput;
					} else if (tChance < tMax) {
						for (int j = 0; j < tOutput.stackSize; j++) if (aRandom.nextInt(tMax) < tChance) {
							if (rArray[i] == null) rArray[i] = UT.Stacks.amount(1, tOutput); else rArray[i].stackSize++;
						}
					} else {
						int tIterations = (3*tChance) / (2*tMax), tAmount = tOutput.stackSize;
						tChance /= tIterations;
						for (int j = 0; j < tIterations; j++) if (aRandom.nextInt(tMax) < tChance) {
							if (rArray[i] == null) rArray[i] = tOutput; else rArray[i].stackSize += tAmount;
						}
					}
				} else {
					rArray[i] = tOutput;
				}
			}
		}
		return rArray;
	}
	
	@Deprecated
	public boolean isRecipeInputEqual(boolean aDecreaseStacksizeBySuccess, FluidStack[] aFluidInputs, ItemStack... aInputs) {
		return isRecipeInputEqual(aDecreaseStacksizeBySuccess, F, aFluidInputs, aInputs);
	}
	
	public boolean isRecipeInputEqual(boolean aDecreaseStacksizeBySuccess, boolean aDontCheckStackSizes, FluidStack[] aFluidInputs, ItemStack... aInputs) {
		if (mFluidInputs.length > 0 && (aFluidInputs == null || aFluidInputs.length < 1)) return F;		
		if (mInputs.length > 0 && (aInputs == null || aInputs.length < 1)) return F;
		
		for (FluidStack tFluid : mFluidInputs) if (tFluid != null) {
			boolean temp = T;
			for (FluidStack aFluid : aFluidInputs) if (aFluid != null && aFluid.isFluidEqual(tFluid) && (aDontCheckStackSizes || aFluid.amount >= tFluid.amount)) {temp = F; break;}
			if (temp) return F;
		}
		
		for (ItemStack tStack : mInputs) if (UT.Stacks.valid(tStack)) {boolean temp = T;
			for (ItemStack aStack : aInputs) if (UT.Stacks.valid(aStack) && OreDictManager.INSTANCE.equal_(F, aStack, tStack) && (aDontCheckStackSizes || aStack.stackSize >= tStack.stackSize)) {temp = F; break;}
			if (temp) return F;
		}
		
		if (aDecreaseStacksizeBySuccess) {
			if (aFluidInputs	!= null) for (FluidStack	tFluid : mFluidInputs	) if (tFluid != null)			for (FluidStack	aFluid : aFluidInputs	) if (aFluid != null && aFluid.isFluidEqual(tFluid)												&& (aDontCheckStackSizes || aFluid.amount		>= tFluid.amount	)) {aFluid.amount		-= tFluid.amount	; break;}
			if (aInputs			!= null) for (ItemStack		tStack : mInputs		) if (UT.Stacks.valid(tStack))	for (ItemStack	aStack : aInputs		) if (UT.Stacks.valid(aStack) && OreDictManager.INSTANCE.equal_(F, aStack, tStack)	&& (aDontCheckStackSizes || aStack.stackSize	>= tStack.stackSize	)) {aStack.stackSize	-= tStack.stackSize	; break;}
		}
		
		return T;
	}
	
	private Recipe(Recipe aRecipe) {
		mInputs = UT.Stacks.copyArray((Object[])aRecipe.mInputs);
		mOutputs = UT.Stacks.copyArray((Object[])aRecipe.mOutputs);
		mSpecialItems = aRecipe.mSpecialItems;
		mChances = Arrays.copyOf(aRecipe.mChances, aRecipe.mChances.length);
		mMaxChances = Arrays.copyOf(aRecipe.mMaxChances, aRecipe.mMaxChances.length);
		mFluidInputs = UT.Fluids.copyArray(aRecipe.mFluidInputs);
		mFluidOutputs = UT.Fluids.copyArray(aRecipe.mFluidOutputs);
		mEUt = aRecipe.mEUt;
		mDuration = aRecipe.mDuration;
		mSpecialValue = aRecipe.mSpecialValue;
		mNeedsEmptyOutput = aRecipe.mNeedsEmptyOutput;
		mCanBeBuffered = aRecipe.mCanBeBuffered;
		mFakeRecipe = aRecipe.mFakeRecipe;
		mEnabled = aRecipe.mEnabled;
		mHidden = aRecipe.mHidden;
	}
	
	protected Recipe(boolean aOptimize, boolean aUnificate, ItemStack[] aInputs, ItemStack[] aOutputs, Object aSpecialItems, long[] aChances, FluidStack[] aFluidInputs, FluidStack[] aFluidOutputs, long aDuration, long aEUt, long aSpecialValue) {
		if (aInputs       == null) aInputs       = ZL_ITEMSTACK;
		if (aOutputs      == null) aOutputs      = ZL_ITEMSTACK;
		if (aFluidInputs  == null) aFluidInputs  = ZL_FLUIDSTACK;
		if (aFluidOutputs == null) aFluidOutputs = ZL_FLUIDSTACK;
		if (aChances      == null) aChances      = aOutputs.length<0?ZL_LONG:new long[aOutputs.length];
		if (aChances.length < aOutputs.length) aChances = Arrays.copyOf(aChances, aOutputs.length);
		aInputs  = UT.Code.getWithoutTrailingNulls(aInputs ).toArray(ZL_ITEMSTACK);
		aOutputs = UT.Code.getWithoutTrailingNulls(aOutputs).toArray(ZL_ITEMSTACK);
		if (aUnificate) {
			OreDictManager.INSTANCE.setStackArray_(T, aInputs );
			OreDictManager.INSTANCE.setStackArray_(T, aOutputs);
		}
		aFluidInputs  = UT.Code.getWithoutNulls(aFluidInputs ).toArray(ZL_FLUIDSTACK);
		aFluidOutputs = UT.Code.getWithoutNulls(aFluidOutputs).toArray(ZL_FLUIDSTACK);
		
		for (ItemStack tStack : aOutputs) {
			if (tStack != null) {
				if (UT.Stacks.meta(tStack) == W) UT.Stacks.meta(tStack, 0);
				UT.Stacks.update(tStack);
			}
		}
		
		for (int i = 0; i < aChances		.length; i++) if (aChances[i] <= 0) aChances[i] = 10000;
		for (int i = 0; i < aFluidInputs	.length; i++) aFluidInputs [i] = aFluidInputs [i].copy();
		for (int i = 0; i < aFluidOutputs	.length; i++) aFluidOutputs[i] = aFluidOutputs[i].copy();
		
		if (aOptimize)  {
			for (int i = 0; i < aInputs.length; i++) if (aInputs[i] != null && UT.Stacks.meta(aInputs[i]) != W) for (int j = 0; j < aOutputs.length; j++) {
				if (UT.Stacks.equal(aInputs[i], aOutputs[j])) {
					if (aInputs[i].stackSize >= aOutputs[j].stackSize) {
						aInputs[i].stackSize -= aOutputs[j].stackSize;
						aOutputs[j] = null;
					} else {
						aOutputs[j].stackSize -= aInputs[i].stackSize;
					}
				}
			}
			
			if (aDuration >= 32) {
				ArrayList<ItemStack> tList = new ArrayListNoNulls<ItemStack>(Arrays.asList(aInputs));
				tList.addAll(Arrays.asList(aOutputs));
				
				for (byte i = (byte)Math.min(64, aDuration / 16); i > 1; i--) if (aDuration / i >= 16) {
					boolean temp = T;
					for (int j = 0, k = tList.size(); temp &&  j < k; j++) if (tList.get(j).stackSize  % i != 0) temp = F;
					for (int j = 0; temp && j < aFluidInputs .length; j++) if (aFluidInputs [j].amount % i != 0) temp = F;
					for (int j = 0; temp && j < aFluidOutputs.length; j++) if (aFluidOutputs[j].amount % i != 0) temp = F;
					if (temp) {
						for (int j = 0, k = tList.size();  j < k; j++) tList.get(j).stackSize  /= i;
						for (int j = 0; j < aFluidInputs .length; j++) aFluidInputs [j].amount /= i;
						for (int j = 0; j < aFluidOutputs.length; j++) aFluidOutputs[j].amount /= i;
						aDuration /= i;
						break;
					}
				}
			}
		}
		
		mInputs = aInputs;
		mOutputs = aOutputs;
		mSpecialItems = aSpecialItems;
		mChances = aChances;
		Arrays.fill(mMaxChances = new long[aChances.length], 10000);
		mFluidInputs = aFluidInputs;
		mFluidOutputs = aFluidOutputs;
		mDuration = aDuration;
		mSpecialValue = aSpecialValue;
		mEUt = aEUt;
		
//		checkCellBalance();
	}
	
	// -----------------------------------------------------------------------------------------------------------------
	// Here are a few Classes I use for Special Cases in some Machines without having to write a separate Machine Class.
	// -----------------------------------------------------------------------------------------------------------------
	
	/**
	 * Abstract Class for general Recipe Handling of non GT Recipes
	 */
	public static abstract class RecipeMapNonGTRecipes extends RecipeMap {
		public RecipeMapNonGTRecipes(Collection<Recipe> aRecipeList, String aUnlocalizedName, String aNameLocal, String aNameNEI, long aProgressBarDirection, long aProgressBarAmount, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aInputFluidCount, long aOutputFluidCount, long aMinimalInputFluids, long aMinimalInputs, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed, boolean aConfigAllowed, boolean aNeedsOutputs) {
			super(aRecipeList, aUnlocalizedName, aNameLocal, aNameNEI, aProgressBarDirection, aProgressBarAmount, aNEIGUIPath, aInputItemsCount, aOutputItemsCount, aMinimalInputItems, aInputFluidCount, aOutputFluidCount, aMinimalInputFluids, aMinimalInputs, aPower, aNEISpecialValuePre, aNEISpecialValueMultiplier, aNEISpecialValuePost, aShowVoltageAmperageInNEI, aNEIAllowed, aConfigAllowed, aNeedsOutputs);
		}
		
		@Override public boolean containsInput(ItemStack aStack) {return F;}
		@Override public boolean containsInput(FluidStack aFluid) {return F;}
		@Override public boolean containsInput(Fluid aFluid) {return F;}
		
		@Override public Recipe add(Recipe aRecipe) {return null;}
		@Override public void reInit() {/**/}
		@Override protected Recipe addToItemMap(Recipe aRecipe) {return null;}
	}
	
	/**
	 * Just a Recipe Map with Utility specifically for Fuels.
	 */
	public static class RecipeMapFuel extends RecipeMap {
		public RecipeMapFuel(Collection<Recipe> aRecipeList, String aUnlocalizedName, String aNameLocal, String aNameNEI, long aProgressBarDirection, long aProgressBarAmount, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aInputFluidCount, long aOutputFluidCount, long aMinimalInputFluids, long aMinimalInputs, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed, boolean aConfigAllowed, boolean aNeedsOutputs) {
			super(aRecipeList, aUnlocalizedName, aNameLocal, aNameNEI, aProgressBarDirection, aProgressBarAmount, aNEIGUIPath, aInputItemsCount, aOutputItemsCount, aMinimalInputItems, aInputFluidCount, aOutputFluidCount, aMinimalInputFluids, aMinimalInputs, aPower, aNEISpecialValuePre, aNEISpecialValueMultiplier, aNEISpecialValuePost, aShowVoltageAmperageInNEI, aNEIAllowed, aConfigAllowed, aNeedsOutputs);
		}
		
		public Recipe addFuel(ItemStack aInput, ItemStack aOutput, long aFuelValueInEU) {
			return addFuel(aInput, aOutput, null, null, 10000, aFuelValueInEU);
		}
		
		public Recipe addFuel(ItemStack aInput, ItemStack aOutput, long aChance, long aFuelValueInEU) {
			return addFuel(aInput, aOutput, null, null, aChance, aFuelValueInEU);
		}
		
		public Recipe addFuel(FluidStack aFluidInput, FluidStack aFluidOutput, long aFuelValueInEU) {
			return addFuel(null, null, aFluidInput, aFluidOutput, 10000, aFuelValueInEU);
		}
		
		public Recipe addFuel(ItemStack aInput, ItemStack aOutput, FluidStack aFluidInput, FluidStack aFluidOutput, long aFuelValueInEU) {
			return addFuel(aInput, aOutput, aFluidInput, aFluidOutput, 10000, aFuelValueInEU);
		}
		
		public Recipe addFuel(ItemStack aInput, ItemStack aOutput, FluidStack aFluidInput, FluidStack aFluidOutput, long aChance, long aFuelValueInEU) {
			return addRecipe(T, new ItemStack[] {aInput}, new ItemStack[] {aOutput}, null, new long[] {aChance}, new FluidStack[] {aFluidInput}, new FluidStack[] {aFluidOutput}, 0, 0, aFuelValueInEU);
		}
	}
	
	/**
	 * Special Class for Furnace Recipe handling.
	 */
	public static class RecipeMapFurnace extends RecipeMapNonGTRecipes {
		public RecipeMapFurnace(Collection<Recipe> aRecipeList, String aUnlocalizedName, String aNameLocal, String aNameNEI, long aProgressBarDirection, long aProgressBarAmount, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aInputFluidCount, long aOutputFluidCount, long aMinimalInputFluids, long aMinimalInputs, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed, boolean aConfigAllowed, boolean aNeedsOutputs) {
			super(aRecipeList, aUnlocalizedName, aNameLocal, aNameNEI, aProgressBarDirection, aProgressBarAmount, aNEIGUIPath, aInputItemsCount, aOutputItemsCount, aMinimalInputItems, aInputFluidCount, aOutputFluidCount, aMinimalInputFluids, aMinimalInputs, aPower, aNEISpecialValuePre, aNEISpecialValueMultiplier, aNEISpecialValuePost, aShowVoltageAmperageInNEI, aNEIAllowed, aConfigAllowed, aNeedsOutputs);
		}
		
		@Override
		public Recipe findRecipe(IHasWorldAndCoords aTileEntity, Recipe aRecipe, boolean aNotUnificated, long aSize, ItemStack aSpecialSlot, FluidStack[] aFluids, ItemStack... aInputs) {
			if (aInputs == null || aInputs.length <= 0 || aInputs[0] == null) return null;
			if (aRecipe != null && aRecipe.isRecipeInputEqual(F, T, aFluids, aInputs)) return aRecipe;
			ItemStack tOutput = GT_ModHandler.getSmeltingOutput(aInputs[0], F, null);
			return tOutput == null ? null : new Recipe(F, F, new ItemStack[] {UT.Stacks.amount(1, aInputs[0])}, new ItemStack[] {tOutput}, null, null, null, null, EU_PER_SMELT / 200 + (EU_PER_SMELT % 200 == 0 ? 0 : 1), 16, 0);
		}
		
		@Override public boolean containsInput(ItemStack aStack) {return GT_ModHandler.getSmeltingOutput(aStack, F, null) != null;}
	}
	
	/**
	 * Special Class for Microwave Recipe handling.
	 */
	public static class RecipeMapMicrowave extends RecipeMapNonGTRecipes {
		public RecipeMapMicrowave(Collection<Recipe> aRecipeList, String aUnlocalizedName, String aNameLocal, String aNameNEI, long aProgressBarDirection, long aProgressBarAmount, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aInputFluidCount, long aOutputFluidCount, long aMinimalInputFluids, long aMinimalInputs, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed, boolean aConfigAllowed, boolean aNeedsOutputs) {
			super(aRecipeList, aUnlocalizedName, aNameLocal, aNameNEI, aProgressBarDirection, aProgressBarAmount, aNEIGUIPath, aInputItemsCount, aOutputItemsCount, aMinimalInputItems, aInputFluidCount, aOutputFluidCount, aMinimalInputFluids, aMinimalInputs, aPower, aNEISpecialValuePre, aNEISpecialValueMultiplier, aNEISpecialValuePost, aShowVoltageAmperageInNEI, aNEIAllowed, aConfigAllowed, aNeedsOutputs);
		}
		
		@Override
		public Recipe findRecipe(IHasWorldAndCoords aTileEntity, Recipe aRecipe, boolean aNotUnificated, long aSize, ItemStack aSpecialSlot, FluidStack[] aFluids, ItemStack... aInputs) {
			if (aInputs == null || aInputs.length <= 0 || aInputs[0] == null) return null;
			if (aRecipe != null && aRecipe.isRecipeInputEqual(F, T, aFluids, aInputs)) return aRecipe;
			ItemStack tOutput = GT_ModHandler.getSmeltingOutput(aInputs[0], F, null);
			
			if (UT.Stacks.equal(aInputs[0], UT.Stacks.make(Items.book, 1, W))) {
				return new Recipe(F, F, new ItemStack[] {UT.Stacks.amount(1, aInputs[0])}, new ItemStack[] {UT.Books.getBookWithTitle("Manual_Microwave")}, null, null, null, null, 32, 4, 0);
			}
			
			// Check Container Item of Input since it is around the Input, then the Input itself, then Container Item of Output and last check the Output itself
			for (ItemStack tStack : new ItemStack[] {UT.Stacks.container(aInputs[0], T), aInputs[0], UT.Stacks.container(tOutput, T), tOutput}) if (UT.Stacks.valid(tStack)) {
				if (UT.Stacks.equal(tStack, UT.Stacks.make(Blocks.netherrack, 1, W), T)
				 || UT.Stacks.equal(tStack, UT.Stacks.make(Blocks.tnt, 1, W), T)
				 || UT.Stacks.equal(tStack, UT.Stacks.make(Items.egg, 1, W), T)
				 || UT.Stacks.equal(tStack, UT.Stacks.make(Items.firework_charge, 1, W), T)
				 || UT.Stacks.equal(tStack, UT.Stacks.make(Items.fireworks, 1, W), T)
				 || UT.Stacks.equal(tStack, UT.Stacks.make(Items.fire_charge, 1, W), T)
				) {
					if (aTileEntity instanceof TileEntityBase01Root) ((TileEntityBase01Root)aTileEntity).overcharge(aSize * 4, TD.Energy.EU);
					return null;
				}
				
				OreDictItemData tData = OM.anydata_(tStack);
				
				if (tData != null) {
					if (tData.mMaterial != null && tData.mMaterial.mMaterial != null) {
						if (tData.mMaterial.mMaterial.contains(TD.Atomic.METAL) || tData.mMaterial.mMaterial.contains(TD.Properties.EXPLOSIVE)) {
							if (aTileEntity instanceof TileEntityBase01Root) ((TileEntityBase01Root)aTileEntity).overcharge(aSize * 4, TD.Energy.EU);
							return null;
						}
						if (tData.mMaterial.mMaterial.contains(TD.Properties.FLAMMABLE)) {
							if (aTileEntity instanceof TileEntityBase01Root) ((TileEntityBase01Root)aTileEntity).setOnFire();
							return null;
						}
					}
					for (OreDictMaterialStack tMaterial : tData.mByProducts) if (tMaterial != null) {
						if (tMaterial.mMaterial.contains(TD.Atomic.METAL) || tMaterial.mMaterial.contains(TD.Properties.EXPLOSIVE)) {
							if (aTileEntity instanceof TileEntityBase01Root) ((TileEntityBase01Root)aTileEntity).overcharge(aSize * 4, TD.Energy.EU);
							return null;
						}
						if (tMaterial.mMaterial.contains(TD.Properties.FLAMMABLE)) {
							if (aTileEntity instanceof TileEntityBase01Root) ((TileEntityBase01Root)aTileEntity).setOnFire();
							return null;
						}
					}
				}
				if (UT.Stacks.fuel(tStack) > 0) {
					if (aTileEntity instanceof TileEntityBase01Root) ((TileEntityBase01Root)aTileEntity).setOnFire();
					return null;
				}
				
			}
			
			return tOutput == null ? null : new Recipe(F, F, new ItemStack[] {UT.Stacks.amount(1, aInputs[0])}, new ItemStack[] {tOutput}, null, null, null, null, 32, 4, 0);
		}
		
		@Override public boolean containsInput(ItemStack aStack) {return GT_ModHandler.getSmeltingOutput(aStack, F, null) != null;}
	}
	
	/**
	 * Special Class for Unboxinator handling.
	 */
	public static class RecipeMapUnboxinator extends RecipeMap {
		public RecipeMapUnboxinator(Collection<Recipe> aRecipeList, String aUnlocalizedName, String aNameLocal, String aNameNEI, long aProgressBarDirection, long aProgressBarAmount, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aInputFluidCount, long aOutputFluidCount, long aMinimalInputFluids, long aMinimalInputs, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed, boolean aConfigAllowed, boolean aNeedsOutputs) {
			super(aRecipeList, aUnlocalizedName, aNameLocal, aNameNEI, aProgressBarDirection, aProgressBarAmount, aNEIGUIPath, aInputItemsCount, aOutputItemsCount, aMinimalInputItems, aInputFluidCount, aOutputFluidCount, aMinimalInputFluids, aMinimalInputs, aPower, aNEISpecialValuePre, aNEISpecialValueMultiplier, aNEISpecialValuePost, aShowVoltageAmperageInNEI, aNEIAllowed, aConfigAllowed, aNeedsOutputs);
		}
		
		@Override
		public Recipe findRecipe(IHasWorldAndCoords aTileEntity, Recipe aRecipe, boolean aNotUnificated, long aSize, ItemStack aSpecialSlot, FluidStack[] aFluids, ItemStack... aInputs) {
			if (aInputs == null || aInputs.length <= 0 || GT_API.sCompatIC2 == null || !IL.IC2_Scrapbox.equal(aInputs[0], F, T)) return super.findRecipe(aTileEntity, aRecipe, aNotUnificated, aSize, aSpecialSlot, aFluids, aInputs);
			ItemStack tOutput = GT_API.sCompatIC2.scrapbox(aInputs[0]);
			if (tOutput == null) return super.findRecipe(aTileEntity, aRecipe, aNotUnificated, aSize, aSpecialSlot, aFluids, aInputs);
			Recipe rRecipe = new Recipe(F, F, new ItemStack[] {IL.IC2_Scrapbox.get(1)}, new ItemStack[] {tOutput}, null, null, null, null, 16, 1, 0);
			// It is not allowed to be buffered due to the random Output
			rRecipe.mCanBeBuffered = F;
			// Due to its randomness it is not good if there are Items in the Output Slot, because those Items could manipulate the outcome.
			rRecipe.mNeedsEmptyOutput = T;
			return rRecipe;
		}
		
		@Override public boolean containsInput(ItemStack aStack) {return IL.IC2_Scrapbox.equal(aStack, F, T) || super.containsInput(aStack);}
	}
	
	/**
	 * Special Class for Fluid Canner handling.
	 */
	public static class RecipeMapFluidCanner extends RecipeMap {
		public RecipeMapFluidCanner(Collection<Recipe> aRecipeList, String aUnlocalizedName, String aNameLocal, String aNameNEI, long aProgressBarDirection, long aProgressBarAmount, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aInputFluidCount, long aOutputFluidCount, long aMinimalInputFluids, long aMinimalInputs, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed, boolean aConfigAllowed, boolean aNeedsOutputs) {
			super(aRecipeList, aUnlocalizedName, aNameLocal, aNameNEI, aProgressBarDirection, aProgressBarAmount, aNEIGUIPath, aInputItemsCount, aOutputItemsCount, aMinimalInputItems, aInputFluidCount, aOutputFluidCount, aMinimalInputFluids, aMinimalInputs, aPower, aNEISpecialValuePre, aNEISpecialValueMultiplier, aNEISpecialValuePost, aShowVoltageAmperageInNEI, aNEIAllowed, aConfigAllowed, aNeedsOutputs);
			mMaxFluidInputSize = 64000;
		}
		
		@Override
		public Recipe findRecipe(IHasWorldAndCoords aTileEntity, Recipe aRecipe, boolean aNotUnificated, long aSize, ItemStack aSpecialSlot, FluidStack[] aFluids, ItemStack... aInputs) {
			Recipe rRecipe = super.findRecipe(aTileEntity, aRecipe, aNotUnificated, aSize, aSpecialSlot, aFluids, aInputs);
			if (aInputs == null || aInputs.length <= 0 || aInputs[0] == null || rRecipe != null || GAPI_POST.mFinishedServerStarted <= 0) return rRecipe;
			if (aFluids != null && aFluids.length >  0 && aFluids[0] != null) {
				ItemStack tOutput = UT.Fluids.fillFluidContainer(aFluids[0], aInputs[0], F, T, F);
				FluidStack tFluid = UT.Fluids.getFluidForFilledItem(tOutput, T);
				if (tFluid != null) rRecipe = new Recipe(F, F, new ItemStack[] {UT.Stacks.amount(1, aInputs[0])}, new ItemStack[] {tOutput}, null, null, new FluidStack[] {tFluid}, null, Math.max(tFluid.amount / 64, 16), 16, 0);
			}
			if (rRecipe == null) {
				FluidStack tFluid = UT.Fluids.getFluidForFilledItem(aInputs[0], T);
				if (tFluid != null) rRecipe = new Recipe(F, F, new ItemStack[] {UT.Stacks.amount(1, aInputs[0])}, new ItemStack[] {UT.Stacks.container(aInputs[0], T)}, null, null, null, new FluidStack[] {tFluid}, Math.max(tFluid.amount / 64, 16), 16, 0);
			}
			if (rRecipe != null) rRecipe.mCanBeBuffered = F;
			return rRecipe;
		}
		
		@Override public boolean containsInput(ItemStack aStack) {return aStack != null && (super.containsInput(aStack) || (aStack.getItem() instanceof IFluidContainerItem && ((IFluidContainerItem)aStack.getItem()).getCapacity(aStack) > 0));}
		@Override public boolean containsInput(FluidStack aFluid) {return T;}
		@Override public boolean containsInput(Fluid aFluid) {return T;}
	}
	
	/**
	 * Special Class for Recycler Recipe handling.
	 */
	public static class RecipeMapRecycler extends RecipeMapNonGTRecipes {
		public RecipeMapRecycler(Collection<Recipe> aRecipeList, String aUnlocalizedName, String aNameLocal, String aNameNEI, long aProgressBarDirection, long aProgressBarAmount, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aInputFluidCount, long aOutputFluidCount, long aMinimalInputFluids, long aMinimalInputs, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed, boolean aConfigAllowed, boolean aNeedsOutputs) {
			super(aRecipeList, aUnlocalizedName, aNameLocal, aNameNEI, aProgressBarDirection, aProgressBarAmount, aNEIGUIPath, aInputItemsCount, aOutputItemsCount, aMinimalInputItems, aInputFluidCount, aOutputFluidCount, aMinimalInputFluids, aMinimalInputs, aPower, aNEISpecialValuePre, aNEISpecialValueMultiplier, aNEISpecialValuePost, aShowVoltageAmperageInNEI, aNEIAllowed, aConfigAllowed, aNeedsOutputs);
		}
		
		@Override
		public Recipe findRecipe(IHasWorldAndCoords aTileEntity, Recipe aRecipe, boolean aNotUnificated, long aSize, ItemStack aSpecialSlot, FluidStack[] aFluids, ItemStack... aInputs) {
			if (aInputs == null || aInputs.length <= 0 || aInputs[0] == null) return null;
			if (aRecipe != null && aRecipe.isRecipeInputEqual(F, T, aFluids, aInputs)) return aRecipe;
		    return new Recipe(F, F, new ItemStack[] {UT.Stacks.amount(1, aInputs[0])}, GT_API.sCompatIC2 != null && GT_API.sCompatIC2.recycler(UT.Stacks.amount(64, aInputs[0]), 0) == null ? null : new ItemStack[] {IL.IC2_Scrap.get(1)}, null, new long[] {1250}, null, null, 45, 1, 0);
		}
		
		@Override public boolean containsInput(ItemStack aStack) {return GT_API.sCompatIC2 == null || GT_API.sCompatIC2.recycler(UT.Stacks.amount(64, aStack), 0) != null;}
	}
	
	/**
	 * Special Class for Compressor Recipe handling.
	 */
	public static class RecipeMapCompressor extends RecipeMapNonGTRecipes {
		public RecipeMapCompressor(Collection<Recipe> aRecipeList, String aUnlocalizedName, String aNameLocal, String aNameNEI, long aProgressBarDirection, long aProgressBarAmount, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aInputFluidCount, long aOutputFluidCount, long aMinimalInputFluids, long aMinimalInputs, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed, boolean aConfigAllowed, boolean aNeedsOutputs) {
			super(aRecipeList, aUnlocalizedName, aNameLocal, aNameNEI, aProgressBarDirection, aProgressBarAmount, aNEIGUIPath, aInputItemsCount, aOutputItemsCount, aMinimalInputItems, aInputFluidCount, aOutputFluidCount, aMinimalInputFluids, aMinimalInputs, aPower, aNEISpecialValuePre, aNEISpecialValueMultiplier, aNEISpecialValuePost, aShowVoltageAmperageInNEI, aNEIAllowed, aConfigAllowed, aNeedsOutputs);
		}
		
		@Override
		public Recipe findRecipe(IHasWorldAndCoords aTileEntity, Recipe aRecipe, boolean aNotUnificated, long aSize, ItemStack aSpecialSlot, FluidStack[] aFluids, ItemStack... aInputs) {
			if (aInputs == null || aInputs.length <= 0 || aInputs[0] == null) return null;
			if (aRecipe != null && aRecipe.isRecipeInputEqual(F, T, aFluids, aInputs)) return aRecipe;
			ItemStack tComparedInput = UT.Stacks.copy(aInputs[0]);
			ItemStack[] tOutputItems = GT_ModHandler.getMachineOutput(tComparedInput, ic2.api.recipe.Recipes.compressor.getRecipes(), T, new NBTTagCompound(), null, null, null);
		    return UT.Code.containsSomething(tOutputItems) ? new Recipe(F, F, new ItemStack[] {UT.Stacks.amount(aInputs[0].stackSize - tComparedInput.stackSize, aInputs[0])}, tOutputItems, null, null, null, null, 400, 2, 0) : null;
		}
		
		@Override public boolean containsInput(ItemStack aStack) {return UT.Code.containsSomething(GT_ModHandler.getMachineOutput(UT.Stacks.amount(64, aStack), ic2.api.recipe.Recipes.compressor.getRecipes(), F, new NBTTagCompound(), null, null, null));}
	}
	
	/**
	 * Special Class for Extractor Recipe handling.
	 */
	public static class RecipeMapExtractor extends RecipeMapNonGTRecipes {
		public RecipeMapExtractor(Collection<Recipe> aRecipeList, String aUnlocalizedName, String aNameLocal, String aNameNEI, long aProgressBarDirection, long aProgressBarAmount, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aInputFluidCount, long aOutputFluidCount, long aMinimalInputFluids, long aMinimalInputs, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed, boolean aConfigAllowed, boolean aNeedsOutputs) {
			super(aRecipeList, aUnlocalizedName, aNameLocal, aNameNEI, aProgressBarDirection, aProgressBarAmount, aNEIGUIPath, aInputItemsCount, aOutputItemsCount, aMinimalInputItems, aInputFluidCount, aOutputFluidCount, aMinimalInputFluids, aMinimalInputs, aPower, aNEISpecialValuePre, aNEISpecialValueMultiplier, aNEISpecialValuePost, aShowVoltageAmperageInNEI, aNEIAllowed, aConfigAllowed, aNeedsOutputs);
		}
		
		@Override
		public Recipe findRecipe(IHasWorldAndCoords aTileEntity, Recipe aRecipe, boolean aNotUnificated, long aSize, ItemStack aSpecialSlot, FluidStack[] aFluids, ItemStack... aInputs) {
			if (aInputs == null || aInputs.length <= 0 || aInputs[0] == null) return null;
			if (aRecipe != null && aRecipe.isRecipeInputEqual(F, T, aFluids, aInputs)) return aRecipe;
			ItemStack tComparedInput = UT.Stacks.copy(aInputs[0]);
			ItemStack[] tOutputItems = GT_ModHandler.getMachineOutput(tComparedInput, ic2.api.recipe.Recipes.extractor.getRecipes(), T, new NBTTagCompound(), null, null, null);
		    return UT.Code.containsSomething(tOutputItems) ? new Recipe(F, F, new ItemStack[] {UT.Stacks.amount(aInputs[0].stackSize - tComparedInput.stackSize, aInputs[0])}, tOutputItems, null, null, null, null, 400, 2, 0) : null;
		}
		
		@Override public boolean containsInput(ItemStack aStack) {return UT.Code.containsSomething(GT_ModHandler.getMachineOutput(UT.Stacks.amount(64, aStack), ic2.api.recipe.Recipes.extractor.getRecipes(), F, new NBTTagCompound(), null, null, null));}
	}
	
	/**
	 * Special Class for Thermal Centrifuge Recipe handling.
	 */
	public static class RecipeMapThermalCentrifuge extends RecipeMapNonGTRecipes {
		public RecipeMapThermalCentrifuge(Collection<Recipe> aRecipeList, String aUnlocalizedName, String aNameLocal, String aNameNEI, long aProgressBarDirection, long aProgressBarAmount, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aInputFluidCount, long aOutputFluidCount, long aMinimalInputFluids, long aMinimalInputs, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed, boolean aConfigAllowed, boolean aNeedsOutputs) {
			super(aRecipeList, aUnlocalizedName, aNameLocal, aNameNEI, aProgressBarDirection, aProgressBarAmount, aNEIGUIPath, aInputItemsCount, aOutputItemsCount, aMinimalInputItems, aInputFluidCount, aOutputFluidCount, aMinimalInputFluids, aMinimalInputs, aPower, aNEISpecialValuePre, aNEISpecialValueMultiplier, aNEISpecialValuePost, aShowVoltageAmperageInNEI, aNEIAllowed, aConfigAllowed, aNeedsOutputs);
		}
		
		@Override
		public Recipe findRecipe(IHasWorldAndCoords aTileEntity, Recipe aRecipe, boolean aNotUnificated, long aSize, ItemStack aSpecialSlot, FluidStack[] aFluids, ItemStack... aInputs) {
			if (aInputs == null || aInputs.length <= 0 || aInputs[0] == null) return null;
			if (aRecipe != null && aRecipe.isRecipeInputEqual(F, T, aFluids, aInputs)) return aRecipe;
			ItemStack tComparedInput = UT.Stacks.copy(aInputs[0]);
			ItemStack[] tOutputItems = GT_ModHandler.getMachineOutput(tComparedInput, ic2.api.recipe.Recipes.centrifuge.getRecipes(), T, new NBTTagCompound(), null, null, null);
		    return UT.Code.containsSomething(tOutputItems) ? new Recipe(F, F, new ItemStack[] {UT.Stacks.amount(aInputs[0].stackSize - tComparedInput.stackSize, aInputs[0])}, tOutputItems, null, null, null, null, 400, 48, 0) : null;
		}
		
		@Override public boolean containsInput(ItemStack aStack) {return UT.Code.containsSomething(GT_ModHandler.getMachineOutput(UT.Stacks.amount(64, aStack), ic2.api.recipe.Recipes.centrifuge.getRecipes(), F, new NBTTagCompound(), null, null, null));}
	}
	
	/**
	 * Special Class for Ore Washer Recipe handling.
	 */
	public static class RecipeMapOreWasher extends RecipeMapNonGTRecipes {
		public RecipeMapOreWasher(Collection<Recipe> aRecipeList, String aUnlocalizedName, String aNameLocal, String aNameNEI, long aProgressBarDirection, long aProgressBarAmount, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aInputFluidCount, long aOutputFluidCount, long aMinimalInputFluids, long aMinimalInputs, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed, boolean aConfigAllowed, boolean aNeedsOutputs) {
			super(aRecipeList, aUnlocalizedName, aNameLocal, aNameNEI, aProgressBarDirection, aProgressBarAmount, aNEIGUIPath, aInputItemsCount, aOutputItemsCount, aMinimalInputItems, aInputFluidCount, aOutputFluidCount, aMinimalInputFluids, aMinimalInputs, aPower, aNEISpecialValuePre, aNEISpecialValueMultiplier, aNEISpecialValuePost, aShowVoltageAmperageInNEI, aNEIAllowed, aConfigAllowed, aNeedsOutputs);
		}
		
		@Override
		public Recipe findRecipe(IHasWorldAndCoords aTileEntity, Recipe aRecipe, boolean aNotUnificated, long aSize, ItemStack aSpecialSlot, FluidStack[] aFluids, ItemStack... aInputs) {
			if (aInputs == null || aInputs.length <= 0 || aInputs[0] == null || aFluids == null || aFluids.length < 1 || !UT.Fluids.water(aFluids[0])) return null;
			if (aRecipe != null && aRecipe.isRecipeInputEqual(F, T, aFluids, aInputs)) return aRecipe;
			ItemStack tComparedInput = UT.Stacks.copy(aInputs[0]);
			NBTTagCompound aRecipeMetaData = new NBTTagCompound();
			ItemStack[] tOutputItems = GT_ModHandler.getMachineOutput(tComparedInput, ic2.api.recipe.Recipes.oreWashing.getRecipes(), T, aRecipeMetaData, null, null, null);
			return UT.Code.containsSomething(tOutputItems) ? new Recipe(F, F, new ItemStack[] {UT.Stacks.amount(aInputs[0].stackSize - tComparedInput.stackSize, aInputs[0])}, tOutputItems, null, null, new FluidStack[] {new FluidStack(aFluids[0].getFluid(), ((NBTTagCompound)aRecipeMetaData.getTag("return")).getInteger("amount"))}, null, 400, 16, 0) : null;
		}
		
		@Override public boolean containsInput(ItemStack aStack) {return UT.Code.containsSomething(GT_ModHandler.getMachineOutput(UT.Stacks.amount(64, aStack), ic2.api.recipe.Recipes.oreWashing.getRecipes(), F, new NBTTagCompound(), null, null, null));}
		@Override public boolean containsInput(FluidStack aFluid) {return UT.Fluids.water(aFluid);}
		@Override public boolean containsInput(Fluid aFluid) {return UT.Fluids.water(new FluidStack(aFluid, 0));}
	}
	
	/**
	 * Special Class for Macerator/RockCrusher Recipe handling.
	 */
	public static class RecipeMapMacerator extends RecipeMap {
		public RecipeMapMacerator(Collection<Recipe> aRecipeList, String aUnlocalizedName, String aNameLocal, String aNameNEI, long aProgressBarDirection, long aProgressBarAmount, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aInputFluidCount, long aOutputFluidCount, long aMinimalInputFluids, long aMinimalInputs, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed, boolean aConfigAllowed, boolean aNeedsOutputs) {
			super(aRecipeList, aUnlocalizedName, aNameLocal, aNameNEI, aProgressBarDirection, aProgressBarAmount, aNEIGUIPath, aInputItemsCount, aOutputItemsCount, aMinimalInputItems, aInputFluidCount, aOutputFluidCount, aMinimalInputFluids, aMinimalInputs, aPower, aNEISpecialValuePre, aNEISpecialValueMultiplier, aNEISpecialValuePost, aShowVoltageAmperageInNEI, aNEIAllowed, aConfigAllowed, aNeedsOutputs);
		}
		
		@Override
		public Recipe findRecipe(IHasWorldAndCoords aTileEntity, Recipe aRecipe, boolean aNotUnificated, long aSize, ItemStack aSpecialSlot, FluidStack[] aFluids, ItemStack... aInputs) {
			if (aInputs == null || aInputs.length <= 0 || aInputs[0] == null || GAPI_POST.mFinishedServerStarted <= 0) return super.findRecipe(aTileEntity, aRecipe, aNotUnificated, aSize, aSpecialSlot, aFluids, aInputs);
		    aRecipe = super.findRecipe(aTileEntity, aRecipe, aNotUnificated, aSize, aSpecialSlot, aFluids, aInputs);
		    if (aRecipe != null) return aRecipe;
		    
		    try {
		    	List<ItemStack> tRecipeOutputs = mods.railcraft.api.crafting.RailcraftCraftingManager.rockCrusher.getRecipe(UT.Stacks.amount(1, aInputs[0])).getRandomizedOuputs();
		    	if (tRecipeOutputs != null) {
		    		aRecipe = new Recipe(F, F, new ItemStack[] {UT.Stacks.amount(1, aInputs[0])}, tRecipeOutputs.toArray(ZL_ITEMSTACK), null, null, null, null, 800, 2, 0);
		    		aRecipe.mCanBeBuffered = F;
		    		aRecipe.mNeedsEmptyOutput = T;
		    		return aRecipe;
		    	}
		    } catch(NoClassDefFoundError e) {if (D1) ERR.println("Railcraft Not loaded");} catch(NullPointerException e) {/**/}
		    
			ItemStack tComparedInput = UT.Stacks.copy(aInputs[0]);
			ItemStack[] tOutputItems = GT_ModHandler.getMachineOutput(tComparedInput, ic2.api.recipe.Recipes.macerator.getRecipes(), T, new NBTTagCompound(), null, null, null);
			return UT.Code.containsSomething(tOutputItems) ? new Recipe(F, F, new ItemStack[] {UT.Stacks.amount(aInputs[0].stackSize - tComparedInput.stackSize, aInputs[0])}, tOutputItems, null, null, null, null, 400, 2, 0) : null;
		}
		
		@Override public boolean containsInput(ItemStack aStack) {return super.containsInput(aStack) || UT.Code.containsSomething(GT_ModHandler.getMachineOutput(UT.Stacks.amount(64, aStack), ic2.api.recipe.Recipes.macerator.getRecipes(), F, new NBTTagCompound(), null, null, null));}
	}
	
	/**
	 * Special Class for Assembler handling.
	 */
	public static class RecipeMapAssembler extends RecipeMap {
		public RecipeMapAssembler(Collection<Recipe> aRecipeList, String aUnlocalizedName, String aNameLocal, String aNameNEI, long aProgressBarDirection, long aProgressBarAmount, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aInputFluidCount, long aOutputFluidCount, long aMinimalInputFluids, long aMinimalInputs, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed, boolean aConfigAllowed, boolean aNeedsOutputs) {
			super(aRecipeList, aUnlocalizedName, aNameLocal, aNameNEI, aProgressBarDirection, aProgressBarAmount, aNEIGUIPath, aInputItemsCount, aOutputItemsCount, aMinimalInputItems, aInputFluidCount, aOutputFluidCount, aMinimalInputFluids, aMinimalInputs, aPower, aNEISpecialValuePre, aNEISpecialValueMultiplier, aNEISpecialValuePost, aShowVoltageAmperageInNEI, aNEIAllowed, aConfigAllowed, aNeedsOutputs);
		}
		
		@Override
		public Recipe findRecipe(IHasWorldAndCoords aTileEntity, Recipe aRecipe, boolean aNotUnificated, long aSize, ItemStack aSpecialSlot, FluidStack[] aFluids, ItemStack... aInputs) {
			Recipe rRecipe = super.findRecipe(aTileEntity, aRecipe, aNotUnificated, aSize, aSpecialSlot, aFluids, aInputs);
			if (aInputs == null || aInputs.length <= 0 || aInputs[0] == null || rRecipe == null || GAPI_POST.mFinishedServerStarted <= 0) return rRecipe;
			for (ItemStack aInput : aInputs) {
				if (IL.Paper_Printed_Pages.equal(aInput, F, T) || IL.Paper_Printed_Pages_Many.equal(aInput, F, T)) {
					rRecipe = rRecipe.copy();
					rRecipe.mCanBeBuffered = F;
					rRecipe.mOutputs[0].setTagCompound(aInput.getTagCompound());
				}
			}
			return rRecipe;
		}
	}
	
	/**
	 * Special Class for Furnace Fuels. Also does not accept Lava Containers.
	 */
	public static class RecipeMapFurnaceFuel extends RecipeMap {
		public RecipeMapFurnaceFuel(Collection<Recipe> aRecipeList, String aUnlocalizedName, String aNameLocal, String aNameNEI, long aProgressBarDirection, long aProgressBarAmount, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aInputFluidCount, long aOutputFluidCount, long aMinimalInputFluids, long aMinimalInputs, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed, boolean aConfigAllowed, boolean aNeedsOutputs) {
			super(aRecipeList, aUnlocalizedName, aNameLocal + " (1 Smelt = 5000 Units)", aNameNEI, aProgressBarDirection, aProgressBarAmount, aNEIGUIPath, aInputItemsCount, aOutputItemsCount, aMinimalInputItems, aInputFluidCount, aOutputFluidCount, aMinimalInputFluids, aMinimalInputs, aPower, aNEISpecialValuePre, aNEISpecialValueMultiplier, aNEISpecialValuePost, aShowVoltageAmperageInNEI, aNEIAllowed, aConfigAllowed, aNeedsOutputs);
		}
		
		@Override
		public Recipe findRecipe(IHasWorldAndCoords aTileEntity, Recipe aRecipe, boolean aNotUnificated, long aSize, ItemStack aSpecialSlot, FluidStack[] aFluids, ItemStack... aInputs) {
			Recipe rRecipe = super.findRecipe(aTileEntity, aRecipe, aNotUnificated, aSize, aSpecialSlot, aFluids, aInputs);
			if (aInputs == null || aInputs.length < 1 || UT.Stacks.invalid(aInputs[0]) || GAPI_POST.mFinishedServerStarted <= 0) return rRecipe;
			if (rRecipe == null) {
				long tFuelValue = UT.Stacks.fuel(aInputs[0]);
				if (tFuelValue > 0 && !UT.Fluids.containsFluid(aInputs[0], Fluids.lava(0), T)) {
					ItemStack tContainer = UT.Stacks.container(aInputs[0], T);
					if (tContainer == null) {
						OreDictItemData tData = OM.anydata_(aInputs[0]);
						OreDictMaterialStack tMaterial = null;
						if (tData != null) {
							for (OreDictMaterialStack aMaterial : tData.getAllMaterialStacks()) {
								if (tMaterial == null || tMaterial.mAmount <= 0)
									tMaterial = OM.stack(aMaterial.mMaterial.mTargetBurning.mMaterial, UT.Code.units(aMaterial.mAmount, U, aMaterial.mMaterial.mTargetBurning.mAmount, false));
								else if (tMaterial.mMaterial == aMaterial.mMaterial.mTargetBurning.mMaterial)
									tMaterial.mAmount += UT.Code.units(aMaterial.mAmount, U, aMaterial.mMaterial.mTargetBurning.mAmount, false);
							}
						}
						rRecipe = new Recipe(T, F, new ItemStack[] {UT.Stacks.amount(1, aInputs[0])}, new ItemStack[] {OM.dust(tMaterial)}, null, null, null, null, 1, (tFuelValue * EU_PER_SMELT) / TICKS_PER_SMELT, 0);
					} else {
						rRecipe = new Recipe(T, F, new ItemStack[] {UT.Stacks.amount(1, aInputs[0])}, new ItemStack[] {tContainer}, null, null, null, null, 1, (tFuelValue * EU_PER_SMELT) / TICKS_PER_SMELT, 0);
					}
					rRecipe.mCanBeBuffered = (aInputs[0].getTagCompound() == null);
					if (rRecipe.mCanBeBuffered) addRecipe(rRecipe, F, F, T);
				}
			}
			return rRecipe;
		}
		
		@Override public boolean containsInput(ItemStack aStack) {return null != findRecipe(null, null, T, Long.MAX_VALUE, null, ZL_FLUIDSTACK, aStack);}
	}
	
	/**
	 * Special Class for Chiseling.
	 */
	public static class RecipeMapChisel extends RecipeMap {
		public RecipeMapChisel(Collection<Recipe> aRecipeList, String aUnlocalizedName, String aNameLocal, String aNameNEI, long aProgressBarDirection, long aProgressBarAmount, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aInputFluidCount, long aOutputFluidCount, long aMinimalInputFluids, long aMinimalInputs, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed, boolean aConfigAllowed, boolean aNeedsOutputs) {
			super(aRecipeList, aUnlocalizedName, aNameLocal, aNameNEI, aProgressBarDirection, aProgressBarAmount, aNEIGUIPath, aInputItemsCount, aOutputItemsCount, aMinimalInputItems, aInputFluidCount, aOutputFluidCount, aMinimalInputFluids, aMinimalInputs, aPower, aNEISpecialValuePre, aNEISpecialValueMultiplier, aNEISpecialValuePost, aShowVoltageAmperageInNEI, aNEIAllowed, aConfigAllowed, aNeedsOutputs);
		}
		
		@Override
		public Recipe findRecipe(IHasWorldAndCoords aTileEntity, Recipe aRecipe, boolean aNotUnificated, long aSize, ItemStack aSpecialSlot, FluidStack[] aFluids, ItemStack... aInputs) {
			Recipe rRecipe = super.findRecipe(aTileEntity, aRecipe, aNotUnificated, aSize, aSpecialSlot, aFluids, aInputs);
			if (aInputs == null || aInputs.length < 1 || aInputs[0] == null || GAPI_POST.mFinishedServerStarted <= 0) return rRecipe;
			if (rRecipe == null) {
				OreDictItemData tData = OM.anyassociation(aInputs[0]);
				if (tData != null && (tData.mPrefix == OP.blockDust || tData.mPrefix == OP.blockIngot || tData.mPrefix == OP.blockGem || tData.mPrefix == OP.blockPlate || tData.mPrefix == OP.blockSolid)) {
					List<ItemStack> tList = OreDictManager.getOres(tData.toString(), true);
					int tSize = tList.size();
					if (tSize > 0) {
						ItemStack tOutput = UT.Stacks.amount(1, tList.get(0));
						tSize--;
						for (int i = 0; i < tSize; i++) if (UT.Stacks.equal(aInputs[0], tList.get(i))) {tOutput = UT.Stacks.amount(1, tList.get(i+1)); break;}
						rRecipe = new Recipe(F, F, new ItemStack[] {UT.Stacks.amount(1, aInputs[0])}, new ItemStack[] {tOutput}, null, null, null, null, 96, 12, 0);
						rRecipe.mCanBeBuffered = T;
						return rRecipe;
					}
				}
			}
			return rRecipe;
		}
	}
	
	/**
	 * Special Class for Forming Press handling.
	 */
	public static class RecipeMapFormingPress extends RecipeMap {
		public RecipeMapFormingPress(Collection<Recipe> aRecipeList, String aUnlocalizedName, String aNameLocal, String aNameNEI, long aProgressBarDirection, long aProgressBarAmount, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aInputFluidCount, long aOutputFluidCount, long aMinimalInputFluids, long aMinimalInputs, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed, boolean aConfigAllowed, boolean aNeedsOutputs) {
			super(aRecipeList, aUnlocalizedName, aNameLocal, aNameNEI, aProgressBarDirection, aProgressBarAmount, aNEIGUIPath, aInputItemsCount, aOutputItemsCount, aMinimalInputItems, aInputFluidCount, aOutputFluidCount, aMinimalInputFluids, aMinimalInputs, aPower, aNEISpecialValuePre, aNEISpecialValueMultiplier, aNEISpecialValuePost, aShowVoltageAmperageInNEI, aNEIAllowed, aConfigAllowed, aNeedsOutputs);
		}
		
		@Override
		public Recipe findRecipe(IHasWorldAndCoords aTileEntity, Recipe aRecipe, boolean aNotUnificated, long aSize, ItemStack aSpecialSlot, FluidStack[] aFluids, ItemStack... aInputs) {
			Recipe rRecipe = super.findRecipe(aTileEntity, aRecipe, aNotUnificated, aSize, aSpecialSlot, aFluids, aInputs);
			if (aInputs == null || aInputs.length < 2 || aInputs[0] == null || aInputs[1] == null || GAPI_POST.mFinishedServerStarted <= 0) return rRecipe;
			if (rRecipe == null) {
				if (IL.Shape_Mold_Name.equal(aInputs[0], F, T)) {
					ItemStack tOutput = UT.Stacks.amount(1, aInputs[1]);
					tOutput.setStackDisplayName(aInputs[0].getDisplayName());
					rRecipe = new Recipe(F, F, new ItemStack[] {IL.Shape_Mold_Name.get(0), UT.Stacks.amount(1, aInputs[1])}, new ItemStack[] {tOutput}, null, null, null, null, 128, 8, 0);
					rRecipe.mCanBeBuffered = F;
					return rRecipe;
				}
				if (IL.Shape_Mold_Name.equal(aInputs[1], F, T)) {
					ItemStack tOutput = UT.Stacks.amount(1, aInputs[0]);
					tOutput.setStackDisplayName(aInputs[1].getDisplayName());
					rRecipe = new Recipe(F, F, new ItemStack[] {IL.Shape_Mold_Name.get(0), UT.Stacks.amount(1, aInputs[0])}, new ItemStack[] {tOutput}, null, null, null, null, 128, 8, 0);
					rRecipe.mCanBeBuffered = F;
					return rRecipe;
				}
				return null;
			}
			for (ItemStack aMold : aInputs) {
				if (IL.Shape_Mold_Credit.equal(aMold, F, T)) {
					NBTTagCompound tNBT = aMold.getTagCompound();
					if (tNBT == null) tNBT = new NBTTagCompound();
					if (!tNBT.hasKey("credit_security_id")) UT.NBT.setNumber(tNBT, "credit_security_id", System.nanoTime());
					aMold.setTagCompound(tNBT);
					
					rRecipe = rRecipe.copy();
					rRecipe.mCanBeBuffered = F;
					rRecipe.mOutputs[0].setTagCompound(tNBT);
					return rRecipe;
				}
			}
			return rRecipe;
		}
	}
	
	/**
	 * Special Class for Scanner handling.
	 */
	public static class RecipeMapScannerVisuals extends RecipeMap {
		public RecipeMapScannerVisuals(Collection<Recipe> aRecipeList, String aUnlocalizedName, String aNameLocal, String aNameNEI, long aProgressBarDirection, long aProgressBarAmount, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aInputFluidCount, long aOutputFluidCount, long aMinimalInputFluids, long aMinimalInputs, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed, boolean aConfigAllowed, boolean aNeedsOutputs) {
			super(aRecipeList, aUnlocalizedName, aNameLocal, aNameNEI, aProgressBarDirection, aProgressBarAmount, aNEIGUIPath, aInputItemsCount, aOutputItemsCount, aMinimalInputItems, aInputFluidCount, aOutputFluidCount, aMinimalInputFluids, aMinimalInputs, aPower, aNEISpecialValuePre, aNEISpecialValueMultiplier, aNEISpecialValuePost, aShowVoltageAmperageInNEI, aNEIAllowed, aConfigAllowed, aNeedsOutputs);
			mMappings.put(new ItemStackContainer(Items.flint_and_steel, 1, W), UT.Stacks.make(Blocks.fire, 1, 0));
			mMappings.put(new ItemStackContainer(Items.reeds, 1, W), UT.Stacks.make(Blocks.reeds, 1, 0));
			mMappings.put(new ItemStackContainer(Items.snowball, 1, W), UT.Stacks.make(Blocks.snow, 1, 0));
			mMappings.put(new ItemStackContainer(Items.wheat_seeds, 1, W), UT.Stacks.make(Blocks.wheat, 1, 0));
			mMappings.put(new ItemStackContainer(Items.wheat, 1, W), UT.Stacks.make(Blocks.wheat, 1, 7));
			mMappings.put(new ItemStackContainer(Items.carrot, 1, W), UT.Stacks.make(Blocks.carrots, 1, 7));
			mMappings.put(new ItemStackContainer(Items.poisonous_potato, 1, W), UT.Stacks.make(Blocks.potatoes, 1, 0));
			mMappings.put(new ItemStackContainer(Items.potato, 1, W), UT.Stacks.make(Blocks.potatoes, 1, 7));
			mMappings.put(new ItemStackContainer(Items.melon_seeds, 1, W), UT.Stacks.make(Blocks.melon_stem, 1, 0));
			mMappings.put(new ItemStackContainer(Items.melon, 1, W), UT.Stacks.make(Blocks.melon_stem, 1, 7));
			mMappings.put(new ItemStackContainer(Items.pumpkin_seeds, 1, W), UT.Stacks.make(Blocks.pumpkin_stem, 1, 7));
			mMappings.put(new ItemStackContainer(Items.dye, 1, 3), UT.Stacks.make(Blocks.cocoa, 1, 8));
			mMappings.put(new ItemStackContainer(Items.string, 1, W), UT.Stacks.make(Blocks.web, 1, 0));
			mMappings.put(new ItemStackContainer(Items.nether_wart, 1, W), UT.Stacks.make(Blocks.nether_wart, 1, 3));
			mMappings.put(new ItemStackContainer(Items.comparator, 1, W), UT.Stacks.make(Blocks.powered_comparator, 1, 0));
			mMappings.put(new ItemStackContainer(Items.repeater, 1, W), UT.Stacks.make(Blocks.powered_repeater, 1, 0));
			mMappings.put(new ItemStackContainer(Items.bed, 1, W), UT.Stacks.make(Blocks.bed, 1, 0));
			mMappings.put(new ItemStackContainer(Items.iron_door, 1, W), UT.Stacks.make(Blocks.iron_door, 1, 0));
			mMappings.put(new ItemStackContainer(Items.wooden_door, 1, W), UT.Stacks.make(Blocks.wooden_door, 1, 0));
			mMappings.put(new ItemStackContainer(Items.ender_pearl, 1, W), UT.Stacks.make(Blocks.portal, 1, 0));
			mMappings.put(new ItemStackContainer(Items.ender_eye, 1, W), UT.Stacks.make(Blocks.end_portal_frame, 1, 0));
			mMappings.put(new ItemStackContainer(Items.water_bucket, 1, W), UT.Stacks.make(Blocks.water, 1, 0));
			mMappings.put(new ItemStackContainer(Items.lava_bucket, 1, W), UT.Stacks.make(Blocks.lava, 1, 0));
			mMappings.put(new ItemStackContainer(Items.cauldron, 1, W), UT.Stacks.make(Blocks.cauldron, 1, 0));
			mMappings.put(new ItemStackContainer(Items.brewing_stand, 1, W), UT.Stacks.make(Blocks.brewing_stand, 1, 0));
			mMappings.put(new ItemStackContainer(Items.flower_pot, 1, W), UT.Stacks.make(Blocks.flower_pot, 1, 0));
		}
		
		public final ItemStackMap<ItemStackContainer, ItemStack> mMappings = new ItemStackMap<ItemStackContainer, ItemStack>();
		
		@Override
		public Recipe findRecipe(IHasWorldAndCoords aTileEntity, Recipe aRecipe, boolean aNotUnificated, long aSize, ItemStack aSpecialSlot, FluidStack[] aFluids, ItemStack... aInputs) {
			Recipe rRecipe = super.findRecipe(aTileEntity, aRecipe, aNotUnificated, aSize, aSpecialSlot, aFluids, aInputs);
			if (aInputs == null || aInputs.length < 2 || aInputs[0] == null || aInputs[1] == null || GAPI_POST.mFinishedServerStarted <= 0) return rRecipe;
			if (rRecipe == null) {
				ItemStack tUSB = null, tScanned = null;
				for (ItemStack aInput : aInputs) if (UT.Stacks.valid(aInput)) {
					if (UT.Stacks.invalid(tUSB) && OM.is("gt:usbstick1", aInput)) tUSB = aInput; else tScanned = aInput;
					if (UT.Stacks.valid(tUSB) && UT.Stacks.valid(tScanned)) {
						assert tScanned != null; // just to make eclipse shut the fuck up
						if (OM.is_("gt:canvas", tScanned)) {
							if (tScanned.hasTagCompound() && tScanned.getTagCompound().hasKey(NBT_CANVAS_BLOCK)) {
								rRecipe = new Recipe(F, F, new ItemStack[] {UT.Stacks.amount(1, tScanned), UT.Stacks.amount(1, tUSB)}, new ItemStack[] {UT.Stacks.amount(1, tUSB), UT.Stacks.amount(1, tScanned)}, null, null, null, null, 64, 16, 0);
								rRecipe.mCanBeBuffered = F;
								if (!rRecipe.mOutputs[0].hasTagCompound()) rRecipe.mOutputs[0].setTagCompound(new NBTTagCompound());
								NBTTagCompound tNBT = new NBTTagCompound();
								tNBT.setInteger(NBT_CANVAS_BLOCK, tScanned.getTagCompound().getInteger(NBT_CANVAS_BLOCK));
								tNBT.setInteger(NBT_CANVAS_META, tScanned.getTagCompound().getInteger(NBT_CANVAS_META));
								rRecipe.mOutputs[0].getTagCompound().setTag(NBT_USB_DATA, tNBT);
								return rRecipe;
							}
							return rRecipe;
						}
						if ((tScanned.getItem() == ItemsGT.sMultiItemBooks || OM.is_("bookWritten", tScanned) || IL.Paper_Printed_Pages.equals(tScanned) || IL.Paper_Printed_Pages_Many.equals(tScanned)) && UT.Code.stringValid(UT.NBT.getBookTitle(tScanned))) {
							rRecipe = new Recipe(F, F, new ItemStack[] {UT.Stacks.amount(1, tScanned), UT.Stacks.amount(1, tUSB)}, new ItemStack[] {UT.Stacks.amount(1, tUSB), UT.Stacks.amount(1, tScanned)}, null, null, null, null, 512, 16, 0);
							rRecipe.mCanBeBuffered = F;
							if (!rRecipe.mOutputs[0].hasTagCompound()) rRecipe.mOutputs[0].setTagCompound(new NBTTagCompound());
							rRecipe.mOutputs[0].getTagCompound().setTag(NBT_USB_DATA, tScanned.getTagCompound().copy());
							return rRecipe;
						}
						if (tScanned.getItem() == Items.filled_map) {
							rRecipe = new Recipe(F, F, new ItemStack[] {UT.Stacks.amount(1, tScanned), UT.Stacks.amount(1, tUSB)}, new ItemStack[] {UT.Stacks.amount(1, tUSB), UT.Stacks.amount(1, tScanned)}, null, null, null, null, 64, 16, 0);
							rRecipe.mCanBeBuffered = F;
							if (!rRecipe.mOutputs[0].hasTagCompound()) rRecipe.mOutputs[0].setTagCompound(new NBTTagCompound());
							rRecipe.mOutputs[0].getTagCompound().setTag(NBT_USB_DATA, UT.NBT.setMapID(new NBTTagCompound(), UT.Stacks.meta(tScanned)));
							return rRecipe;
						}
						ItemStack tMapping = mMappings.get(tScanned.getItem(), UT.Stacks.meta(tScanned));
						if (tMapping == null) tMapping = mMappings.get(tScanned.getItem(), W);
						Block tBlock = UT.Stacks.block_(tMapping==null?tScanned:tMapping);
						if (tBlock != null && tBlock != NB) {
							rRecipe = new Recipe(F, F, new ItemStack[] {UT.Stacks.amount(1, tScanned), UT.Stacks.amount(1, tUSB)}, new ItemStack[] {UT.Stacks.amount(1, tUSB), UT.Stacks.amount(1, tScanned)}, null, null, null, null, 512, 16, 0);
							rRecipe.mCanBeBuffered = F;
							if (!rRecipe.mOutputs[0].hasTagCompound()) rRecipe.mOutputs[0].setTagCompound(new NBTTagCompound());
							NBTTagCompound tNBT = new NBTTagCompound();
							tNBT.setInteger(NBT_CANVAS_BLOCK, Block.getIdFromBlock(tBlock));
							tNBT.setInteger(NBT_CANVAS_META, UT.Stacks.meta(tScanned));
							rRecipe.mOutputs[0].getTagCompound().setTag(NBT_USB_DATA, tNBT);
							return rRecipe;
						}
						return rRecipe;
					}
				}
			}
			return rRecipe;
		}
		
		@Override public boolean containsInput(ItemStack aStack) {return T;}
	}
	
	/**
	 * Special Class for Printer handling.
	 */
	public static class RecipeMapPrinter extends RecipeMap {
		public RecipeMapPrinter(Collection<Recipe> aRecipeList, String aUnlocalizedName, String aNameLocal, String aNameNEI, long aProgressBarDirection, long aProgressBarAmount, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aInputFluidCount, long aOutputFluidCount, long aMinimalInputFluids, long aMinimalInputs, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed, boolean aConfigAllowed, boolean aNeedsOutputs) {
			super(aRecipeList, aUnlocalizedName, aNameLocal, aNameNEI, aProgressBarDirection, aProgressBarAmount, aNEIGUIPath, aInputItemsCount, aOutputItemsCount, aMinimalInputItems, aInputFluidCount, aOutputFluidCount, aMinimalInputFluids, aMinimalInputs, aPower, aNEISpecialValuePre, aNEISpecialValueMultiplier, aNEISpecialValuePost, aShowVoltageAmperageInNEI, aNEIAllowed, aConfigAllowed, aNeedsOutputs);
		}
		
		@Override
		public Recipe findRecipe(IHasWorldAndCoords aTileEntity, Recipe aRecipe, boolean aNotUnificated, long aSize, ItemStack aSpecialSlot, FluidStack[] aFluids, ItemStack... aInputs) {
			Recipe rRecipe = super.findRecipe(aTileEntity, aRecipe, aNotUnificated, aSize, aSpecialSlot, aFluids, aInputs);
			
			if (aInputs == null || aInputs.length <= 0 || aInputs[0] == null || aFluids == null || aFluids.length <= 0 || aFluids[0] == null || GAPI_POST.mFinishedServerStarted <= 0) return rRecipe;
			
			if (rRecipe == null) {
				ItemStack tUSB = null, tPaper = null;
				for (ItemStack aInput : aInputs) if (aInput != null) {
					if (tUSB == null && OM.is("gt:usbstick1", aInput) && aInput.hasTagCompound()) tUSB = aInput; else tPaper = aInput;
					if (tUSB != null && tPaper != null) {
						NBTTagCompound tData = tUSB.getTagCompound().getCompoundTag(NBT_USB_DATA);
						if (tData == null || tData.hasNoTags()) return rRecipe;
						if (OM.is("gt:canvas", tPaper)) {
							if (tData.hasKey(NBT_CANVAS_BLOCK) && (!tPaper.hasTagCompound() || !tPaper.getTagCompound().hasKey(NBT_CANVAS_BLOCK))) {
								rRecipe = new Recipe(F, F, new ItemStack[] {UT.Stacks.amount(1, tPaper), UT.Stacks.amount(0, tUSB)}, new ItemStack[] {UT.Stacks.amount(1, tPaper)}, null, null, new FluidStack[] {UT.Fluids.mul(DYE_FLUIDS_CHEMICAL[DYE_INDEX_Black], 1, 9, T), UT.Fluids.mul(DYE_FLUIDS_CHEMICAL[DYE_INDEX_Cyan], 1, 9, T), UT.Fluids.mul(DYE_FLUIDS_CHEMICAL[DYE_INDEX_Magenta], 1, 9, T), UT.Fluids.mul(DYE_FLUIDS_CHEMICAL[DYE_INDEX_Yellow], 1, 9, T)}, null, 64, 16, 0);
								rRecipe.mCanBeBuffered = F;
								NBTTagCompound tNBT = rRecipe.mOutputs[0].getTagCompound();
								if (tNBT == null) tNBT = new NBTTagCompound();
								tNBT.setInteger(NBT_CANVAS_BLOCK, tData.getInteger(NBT_CANVAS_BLOCK));
								tNBT.setInteger(NBT_CANVAS_META, tData.getInteger(NBT_CANVAS_META));
								rRecipe.mOutputs[0].setTagCompound(tNBT);
								return rRecipe;
							}
							return rRecipe;
						}
						if (IL.Paper_Punch_Card_Empty.equal(tPaper)) {
							if (UT.Code.stringValid(UT.NBT.getPunchCardData(tData))) {
								rRecipe = new Recipe(F, F, new ItemStack[] {UT.Stacks.amount(1, tPaper), UT.Stacks.amount(0, tUSB)}, new ItemStack[] {IL.Paper_Punch_Card_Encoded.get(1)}, null, null, new FluidStack[] {UT.Fluids.mul(DYE_FLUIDS_CHEMICAL[DYE_INDEX_Black], 1, 9, T)}, null, 32, 16, 0);
								rRecipe.mCanBeBuffered = F;
								UT.NBT.setPunchCardData(rRecipe.mOutputs[0], UT.NBT.getPunchCardData(tData));
								return rRecipe;
							}
							return rRecipe;
						}
						if (OM.is("paperEmpty", tPaper)) {
							if (UT.Code.stringValid(UT.NBT.getBookTitle(tData)) && UT.Code.stringValid(UT.NBT.getBookAuthor(tData))) {
								NBTTagList tPages = tData.getTagList("pages", 8);
								if (tPages == null || tPages.tagCount() < 1) {
							    	String aMapping = UT.NBT.getBookMapping(tData);
							    	if (UT.Code.stringValid(aMapping)) {
							    		ItemStack tBook = UT.Books.getWrittenBook(aMapping, NI);
							    		if (tBook != null && tBook.hasTagCompound()) tPages = tBook.getTagCompound().getTagList("pages", 8);
							    	}
								}
								boolean tUseManyPages = (tPages != null && tPages.tagCount() > 50);
								rRecipe = new Recipe(F, F, new ItemStack[] {UT.Stacks.amount(tUseManyPages?6:3, tPaper), UT.Stacks.amount(0, tUSB)}, new ItemStack[] {tUseManyPages?IL.Paper_Printed_Pages_Many.get(1):IL.Paper_Printed_Pages.get(1)}, null, null, new FluidStack[] {UT.Fluids.mul(DYE_FLUIDS_CHEMICAL[DYE_INDEX_Black], 1, tUseManyPages?1:2, T)}, null, tUseManyPages?1024:512, 16, 0);
								rRecipe.mCanBeBuffered = F;
								rRecipe.mOutputs[0].setTagCompound((NBTTagCompound)tData.copy());
								return rRecipe;
							}
							return rRecipe;
						}
						if (tPaper.getItem() == Items.map) {
							short tMapID = UT.NBT.getMapID(tData);
							if (tMapID >= 0) {
								rRecipe = new Recipe(F, F, new ItemStack[] {UT.Stacks.amount(1, tPaper), UT.Stacks.amount(0, tUSB)}, new ItemStack[] {UT.Stacks.make(Items.filled_map, 1, tMapID)}, null, null, new FluidStack[] {UT.Fluids.mul(DYE_FLUIDS_CHEMICAL[DYE_INDEX_Black], 1, 9, T), UT.Fluids.mul(DYE_FLUIDS_CHEMICAL[DYE_INDEX_Cyan], 1, 9, T), UT.Fluids.mul(DYE_FLUIDS_CHEMICAL[DYE_INDEX_Magenta], 1, 9, T), UT.Fluids.mul(DYE_FLUIDS_CHEMICAL[DYE_INDEX_Yellow], 1, 9, T)}, null, 64, 16, 0);
								rRecipe.mCanBeBuffered = F;
								return rRecipe;
							}
							return rRecipe;
						}
						return rRecipe;
					}
				}
			}
			/*
			if (rRecipe == null) {
				ItemStack
				tOutput = GT_ModHandler.getAllRecipeOutput(aTileEntity==null?null:aTileEntity.getWorld(), aInputs[0], aInputs[0], aInputs[0], aInputs[0], IL.DYE_ONLY_ITEMS[aDye.mIndex].get(1), aInputs[0], aInputs[0], aInputs[0], aInputs[0]);
				if (tOutput != null) return addRecipe(new Recipe(T, new ItemStack[] {UT.Stacks.amount_(8, aInputs[0])}, new ItemStack[] {tOutput}, null, null, new FluidStack[] {new FluidStack(aFluids[0].getFluid(), (int)L)}, null, 256, 2, 0), F, F, T);
				
				tOutput = GT_ModHandler.getAllRecipeOutput(aTileEntity==null?null:aTileEntity.getWorld(), aInputs[0], IL.DYE_ONLY_ITEMS[aDye.mIndex].get(1));
				if (tOutput != null) return addRecipe(new Recipe(T, new ItemStack[] {UT.Stacks.amount_(1, aInputs[0])}, new ItemStack[] {tOutput}, null, null, new FluidStack[] {new FluidStack(aFluids[0].getFluid(), (int)L)}, null,  32, 2, 0), F, F, T);
			}*/
			return rRecipe;
		}
		
		@Override public boolean containsInput(ItemStack aStack) {return T;}
	}
	
	/**
	 * Special Abstract Class for dynamic Recipe handling.
	 */
	public static abstract class RecipeMapSpecialSingleInput extends RecipeMap {
		public RecipeMapSpecialSingleInput(Collection<Recipe> aRecipeList, String aUnlocalizedName, String aNameLocal, String aNameNEI, long aProgressBarDirection, long aProgressBarAmount, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aInputFluidCount, long aOutputFluidCount, long aMinimalInputFluids, long aMinimalInputs, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed, boolean aConfigAllowed, boolean aNeedsOutputs) {
			super(aRecipeList, aUnlocalizedName, aNameLocal, aNameNEI, aProgressBarDirection, aProgressBarAmount, aNEIGUIPath, aInputItemsCount, aOutputItemsCount, aMinimalInputItems, aInputFluidCount, aOutputFluidCount, aMinimalInputFluids, aMinimalInputs, aPower, aNEISpecialValuePre, aNEISpecialValueMultiplier, aNEISpecialValuePost, aShowVoltageAmperageInNEI, aNEIAllowed, aConfigAllowed, aNeedsOutputs);
		}
		
		@Override
		public Recipe findRecipe(IHasWorldAndCoords aTileEntity, Recipe aRecipe, boolean aNotUnificated, long aSize, ItemStack aSpecialSlot, FluidStack[] aFluids, ItemStack... aInputs) {
			Recipe rRecipe = super.findRecipe(aTileEntity, aRecipe, aNotUnificated, aSize, aSpecialSlot, aFluids, aInputs);
			if (rRecipe != null || aInputs == null || aInputs.length <= 0 || aInputs[0] == null || GAPI_POST.mFinishedServerStarted <= 0) return rRecipe;
			return getRecipeFor(aInputs[0]);
		}
		
		@Override
		public boolean containsInput(ItemStack aStack) {
			return super.containsInput(aStack) || getRecipeFor(aStack) != null;
		}
		
		@Override
		public List<Recipe> getNEIUsages(ItemStack... aInputs) {
			List<Recipe> rList = super.getNEIUsages(aInputs);
			if (rList.isEmpty()) for (ItemStack aInput : aInputs) if (rList.add(getRecipeFor(aInput))) break;
			return rList;
		}
		
		protected abstract Recipe getRecipeFor(ItemStack aInput);
	}
	
	/**
	 * Special Class for Shredder Recipe handling, because Recycling.
	 */
	public static class RecipeMapShredder extends RecipeMapSpecialSingleInput {
		public RecipeMapShredder(Collection<Recipe> aRecipeList, String aUnlocalizedName, String aNameLocal, String aNameNEI, long aProgressBarDirection, long aProgressBarAmount, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aInputFluidCount, long aOutputFluidCount, long aMinimalInputFluids, long aMinimalInputs, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed, boolean aConfigAllowed, boolean aNeedsOutputs) {
			super(aRecipeList, aUnlocalizedName, aNameLocal, aNameNEI, aProgressBarDirection, aProgressBarAmount, aNEIGUIPath, aInputItemsCount, aOutputItemsCount, aMinimalInputItems, aInputFluidCount, aOutputFluidCount, aMinimalInputFluids, aMinimalInputs, aPower, aNEISpecialValuePre, aNEISpecialValueMultiplier, aNEISpecialValuePost, aShowVoltageAmperageInNEI, aNEIAllowed, aConfigAllowed, aNeedsOutputs);
		}
		
		private List<Recipe> mBufferedDynamicRecipes = null;
		
		@Override
		public List<Recipe> getNEIAllRecipes() {
			List<Recipe> rList = super.getNEIAllRecipes();
			if (mBufferedDynamicRecipes == null) {
				mBufferedDynamicRecipes = new ArrayListNoNulls<Recipe>();
				for (OreDictMaterial tMaterial : OP.dust.mRegisteredMaterials) {
					for (ItemStackContainer tStack : tMaterial.mRegisteredItems) {
						mBufferedDynamicRecipes.add(getRecipeFor(tStack.toStack()));
					}
				}
			}
			rList.addAll(mBufferedDynamicRecipes);
			return rList;
		}
		
		@Override
		public List<Recipe> getNEIRecipes(ItemStack... aOutputs) {
			List<Recipe> rList = super.getNEIRecipes(aOutputs);
			for (ItemStack aOutput : aOutputs) {
				OreDictItemData aData = OM.anyassociation(aOutput);
				if (aData != null && aData.mPrefix.contains(TD.Prefix.DUST_BASED)) {
					for (ItemStackContainer tStack : aData.mMaterial.mMaterial.mRegisteredItems) {
						rList.add(getRecipeFor(tStack.toStack()));
					}
					break;
				}
			}
			return rList;
		}
		
		@Override
		protected Recipe getRecipeFor(ItemStack aInput) {
			OreDictItemData aData = OM.anydata(aInput);
			if (aData == null || (aData.mMaterial != null && aData.mMaterial.mMaterial.contains(TD.Atomic.ANTIMATTER)) || (aData.mPrefix != null && (UT.Fluids.getFluidForFilledItem(aInput, T) != null || aData.mPrefix.contains(TD.Prefix.DUST_BASED) || aData.mPrefix.contains(TD.Prefix.ORE) || !aData.mPrefix.contains(TD.Prefix.RECYCLABLE)))) return null;
			List<OreDictMaterialStack> tList = new ArrayListNoNulls<OreDictMaterialStack>();
			for (OreDictMaterialStack tMaterial : aData.getAllMaterialStacks()) if (tMaterial.mMaterial.mTargetPulver.mAmount > 0) OM.stack(UT.Code.units(tMaterial.mAmount, U, tMaterial.mMaterial.mTargetPulver.mAmount, F), tMaterial.mMaterial.mTargetPulver.mMaterial).addToList(tList);
			if (tList.isEmpty()) return null;
			ItemStack[] tOutputs = new ItemStack[Math.min(tList.size(), mOutputItemsCount)];
			int i = 0, tDuration = 0;
			for (OreDictMaterialStack tMaterial : tList) {
				tDuration += UT.Code.units(tMaterial.mAmount, U, 256*Math.max(1, tMaterial.mMaterial.mToolQuality+1), T);
				if (i < tOutputs.length) {
					ItemStack tStack = OM.dust(tMaterial);
					if (tStack != null) tOutputs[i++] = tStack;
				}
			}
			if (!UT.Code.exists(0, tOutputs)) return null;
			Recipe rRecipe = new Recipe(F, F, new ItemStack[] {UT.Stacks.amount(1, aInput)}, tOutputs, null, null, ZL_FLUIDSTACK, ZL_FLUIDSTACK, Math.max(1, tDuration), 16, 0);
			rRecipe.mCanBeBuffered = F;
			return rRecipe;
		}
	}
	
	/**
	 * Special Class for Crusher Recipe handling, because there are just way too many Ores.
	 */
	public static class RecipeMapCrusher extends RecipeMapSpecialSingleInput {
		public RecipeMapCrusher(Collection<Recipe> aRecipeList, String aUnlocalizedName, String aNameLocal, String aNameNEI, long aProgressBarDirection, long aProgressBarAmount, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aInputFluidCount, long aOutputFluidCount, long aMinimalInputFluids, long aMinimalInputs, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed, boolean aConfigAllowed, boolean aNeedsOutputs) {
			super(aRecipeList, aUnlocalizedName, aNameLocal, aNameNEI, aProgressBarDirection, aProgressBarAmount, aNEIGUIPath, aInputItemsCount, aOutputItemsCount, aMinimalInputItems, aInputFluidCount, aOutputFluidCount, aMinimalInputFluids, aMinimalInputs, aPower, aNEISpecialValuePre, aNEISpecialValueMultiplier, aNEISpecialValuePost, aShowVoltageAmperageInNEI, aNEIAllowed, aConfigAllowed, aNeedsOutputs);
		}
		
		private List<Recipe> mBufferedDynamicRecipes = null;
		
		@Override
		public List<Recipe> getNEIAllRecipes() {
			List<Recipe> rList = super.getNEIAllRecipes();
			if (mBufferedDynamicRecipes == null) {
				mBufferedDynamicRecipes = new ArrayListNoNulls<Recipe>();
				for (OreDictMaterial tMaterial : OP.crushed.mRegisteredMaterials) {
					for (ItemStackContainer tStack : tMaterial.mRegisteredItems) {
						mBufferedDynamicRecipes.add(getRecipeFor(tStack.toStack()));
					}
				}
			}
			rList.addAll(mBufferedDynamicRecipes);
			return rList;
		}
		
		@Override
		public List<Recipe> getNEIRecipes(ItemStack... aOutputs) {
			List<Recipe> rList = super.getNEIRecipes(aOutputs);
			for (ItemStack aOutput : aOutputs) {
				OreDictItemData aData = OM.anyassociation(aOutput);
				if (aData != null && aData.mPrefix == OP.crushed) {
					for (ItemStackContainer tStack : aData.mMaterial.mMaterial.mRegisteredItems) {
						rList.add(getRecipeFor(tStack.toStack()));
					}
					break;
				}
			}
			return rList;
		}
		
		@Override
		protected Recipe getRecipeFor(ItemStack aInput) {
			OreDictItemData aData = OM.anydata(aInput);
			if (aData == null || (aData.mMaterial != null && aData.mMaterial.mMaterial.contains(TD.Atomic.ANTIMATTER)) || (aData.mPrefix == null || !aData.mPrefix.contains(TD.Prefix.ORE) || aData.mPrefix.contains(TD.Prefix.DUST_ORE))) return null;
			if (aData.mPrefix == orePoor || aData.mPrefix == oreSmall || aData.mPrefix == oreRich || aData.mPrefix == oreNormal) {
				// TODO
				return null;
			}
			ItemStack[] tOutputs = new ItemStack[Math.max(1, mOutputItemsCount)];
			tOutputs[0] = OP.crushed.mat(aData.mMaterial.mMaterial.mTargetCrushing.mMaterial, UT.Code.units(aData.mMaterial.mMaterial.mTargetCrushing.mAmount, U, 2 * aData.mMaterial.mMaterial.mOreMultiplier * Math.max(1, ConfigsGT.OREPROCESSING.get(ConfigCategories.Materials.oreprocessingoutputmultiplier, aData.mMaterial.toString(), 1L)) * (aData.mPrefix == OP.oreNether || aData.mPrefix == OP.oreDense || aData.mPrefix == OP.oreEnd ? 2 : 1), F));
			if (tOutputs[0] == null) return null;
			int i = 0, tDuration = 200*tOutputs[0].stackSize*Math.max(1, aData.mMaterial.mMaterial.mToolQuality+1);
			for (OreDictMaterialStack tMaterial : aData.mPrefix.mByProducts) {
				tDuration += UT.Code.units(tMaterial.mAmount, U, 200*Math.max(1, tMaterial.mMaterial.mToolQuality+1), T);
				if (i < tOutputs.length - 1) {
					ItemStack tStack = OM.dust(tMaterial.mMaterial.mTargetCrushing.mMaterial, UT.Code.units(tMaterial.mAmount, U, tMaterial.mMaterial.mTargetCrushing.mAmount, F));
					if (tStack != null) tOutputs[++i] = tStack;
				}
			}
			Recipe rRecipe = new Recipe(F, F, new ItemStack[] {UT.Stacks.amount(1, aInput)}, tOutputs, null, null, ZL_FLUIDSTACK, ZL_FLUIDSTACK, Math.max(1, tDuration), 16, 0);
			rRecipe.mCanBeBuffered = F;
			return rRecipe;
		}
	}
	
	/**
	 * Special Class for Hammer Recipe handling, because there are just way too many Ores.
	 */
	public static class RecipeMapHammer extends RecipeMapSpecialSingleInput {
		public RecipeMapHammer(Collection<Recipe> aRecipeList, String aUnlocalizedName, String aNameLocal, String aNameNEI, long aProgressBarDirection, long aProgressBarAmount, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aInputFluidCount, long aOutputFluidCount, long aMinimalInputFluids, long aMinimalInputs, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed, boolean aConfigAllowed, boolean aNeedsOutputs) {
			super(aRecipeList, aUnlocalizedName, aNameLocal, aNameNEI, aProgressBarDirection, aProgressBarAmount, aNEIGUIPath, aInputItemsCount, aOutputItemsCount, aMinimalInputItems, aInputFluidCount, aOutputFluidCount, aMinimalInputFluids, aMinimalInputs, aPower, aNEISpecialValuePre, aNEISpecialValueMultiplier, aNEISpecialValuePost, aShowVoltageAmperageInNEI, aNEIAllowed, aConfigAllowed, aNeedsOutputs);
		}
		
		private List<Recipe> mBufferedDynamicRecipes = null;
		
		@Override
		public List<Recipe> getNEIAllRecipes() {
			List<Recipe> rList = super.getNEIAllRecipes();
			if (mBufferedDynamicRecipes == null) {
				mBufferedDynamicRecipes = new ArrayListNoNulls<Recipe>();
				for (OreDictMaterial tMaterial : OP.crushed.mRegisteredMaterials) {
					for (ItemStackContainer tStack : tMaterial.mRegisteredItems) {
						mBufferedDynamicRecipes.add(getRecipeFor(tStack.toStack()));
					}
				}
			}
			rList.addAll(mBufferedDynamicRecipes);
			return rList;
		}
		
		@Override
		public List<Recipe> getNEIRecipes(ItemStack... aOutputs) {
			List<Recipe> rList = super.getNEIRecipes(aOutputs);
			for (ItemStack aOutput : aOutputs) {
				OreDictItemData aData = OM.anyassociation(aOutput);
				if (aData != null && aData.mPrefix == OP.crushed) {
					for (ItemStackContainer tStack : aData.mMaterial.mMaterial.mRegisteredItems) {
						rList.add(getRecipeFor(tStack.toStack()));
					}
					break;
				}
			}
			return rList;
		}
		
		@Override
		protected Recipe getRecipeFor(ItemStack aInput) {
			OreDictItemData aData = OM.anydata(aInput);
			if (aData == null || (aData.mMaterial != null && aData.mMaterial.mMaterial.contains(TD.Atomic.ANTIMATTER)) || (aData.mPrefix == null || !aData.mPrefix.contains(TD.Prefix.ORE) || aData.mPrefix.contains(TD.Prefix.DUST_ORE))) return null;
			if (aData.mPrefix == orePoor || aData.mPrefix == oreSmall || aData.mPrefix == oreRich || aData.mPrefix == oreNormal) {
				// TODO
				return null;
			}
			ItemStack[] tOutputs = new ItemStack[Math.max(1, mOutputItemsCount)];
			tOutputs[0] = OP.crushed.mat(aData.mMaterial.mMaterial.mTargetCrushing.mMaterial, UT.Code.units(aData.mMaterial.mMaterial.mTargetCrushing.mAmount, U, aData.mMaterial.mMaterial.mOreMultiplier * Math.max(1, ConfigsGT.OREPROCESSING.get(ConfigCategories.Materials.oreprocessingoutputmultiplier, aData.mMaterial.toString(), 1L)) * (aData.mPrefix == OP.oreNether || aData.mPrefix == OP.oreDense || aData.mPrefix == OP.oreEnd ? 2 : 1), F));
			if (tOutputs[0] == null) return null;
			int i = 0, tDuration = 16*tOutputs[0].stackSize*Math.max(1, aData.mMaterial.mMaterial.mToolQuality+1);
			for (OreDictMaterialStack tMaterial : aData.mPrefix.mByProducts) {
				tDuration += UT.Code.units(tMaterial.mAmount, U, 200*Math.max(1, tMaterial.mMaterial.mToolQuality+1), T);
				if (i < tOutputs.length - 1) {
					ItemStack tStack = OM.dust(tMaterial.mMaterial.mTargetCrushing.mMaterial, UT.Code.units(tMaterial.mAmount, U, tMaterial.mMaterial.mTargetCrushing.mAmount, F));
					if (tStack != null) tOutputs[++i] = tStack;
				}
			}
			Recipe rRecipe = new Recipe(F, F, new ItemStack[] {UT.Stacks.amount(1, aInput)}, tOutputs, null, null, ZL_FLUIDSTACK, ZL_FLUIDSTACK, Math.max(1, tDuration), 16, 0);
			rRecipe.mCanBeBuffered = F;
			return rRecipe;
		}
	}
	
	/**
	 * Special Class for Cutter Recipe handling, because someone at Forestry got the retarded Idea of using NBT INSTEAD of MetaData for its Wood Types...
	 */
	public static class RecipeMapCutter extends RecipeMap {
		public RecipeMapCutter(Collection<Recipe> aRecipeList, String aUnlocalizedName, String aNameLocal, String aNameNEI, long aProgressBarDirection, long aProgressBarAmount, String aNEIGUIPath, long aInputItemsCount, long aOutputItemsCount, long aMinimalInputItems, long aInputFluidCount, long aOutputFluidCount, long aMinimalInputFluids, long aMinimalInputs, long aPower, String aNEISpecialValuePre, long aNEISpecialValueMultiplier, String aNEISpecialValuePost, boolean aShowVoltageAmperageInNEI, boolean aNEIAllowed, boolean aConfigAllowed, boolean aNeedsOutputs) {
			super(aRecipeList, aUnlocalizedName, aNameLocal, aNameNEI, aProgressBarDirection, aProgressBarAmount, aNEIGUIPath, aInputItemsCount, aOutputItemsCount, aMinimalInputItems, aInputFluidCount, aOutputFluidCount, aMinimalInputFluids, aMinimalInputs, aPower, aNEISpecialValuePre, aNEISpecialValueMultiplier, aNEISpecialValuePost, aShowVoltageAmperageInNEI, aNEIAllowed, aConfigAllowed, aNeedsOutputs);
		}
		/*
		@Override
		public Recipe findRecipe(IHasWorldAndCoords aTileEntity, Recipe aRecipe, boolean aNotUnificated, long aSize, ItemStack aSpecialSlot, FluidStack[] aFluids, ItemStack... aInputs) {
			Recipe rRecipe = super.findRecipe(aTileEntity, aRecipe, aNotUnificated, aSize, aSpecialSlot, aFluids, aInputs);
			
			if (rRecipe != null || aInputs == null || aInputs.length <= 0 || aInputs[0] == null || aFluids == null || aFluids.length <= 0 || aFluids[0] == null || !API.mFinishedServerStarted) return rRecipe;
			
			if (IL.FR_Logs.equal(aInputs[0], T, T)) {
				if (UT.Fluids.equal(MT.Lubricant.liquid(U, T), aFluids[0]))	rRecipe = new Recipe(F, new ItemStack[] {UT.Stacks.amount_(1, aInputs[0])}, new ItemStack[] {UT.Stacks.create(IL.FR_Planks.getWithDamage(              6, UT.Stacks.meta(aInputs[0])), aInputs[0].getTagCompound()), dust.mat(MT.Wood, 1)}, null, null, new FluidStack[] {MT.Lubricant.liquid(U/200, T)}, ZL_FLUIDSTACK, 128, 16, 0); else
				if (UT.Fluids.equal(MT.Water.liquid(U, T), aFluids[0]))		rRecipe = new Recipe(F, new ItemStack[] {UT.Stacks.amount_(1, aInputs[0])}, new ItemStack[] {UT.Stacks.create(IL.FR_Planks.getWithDamage(NERFED_WOOD?4:5, UT.Stacks.meta(aInputs[0])), aInputs[0].getTagCompound()), dust.mat(MT.Wood, 2)}, null, null, new FluidStack[] {MT.Water    .liquid(U/ 50, T)}, ZL_FLUIDSTACK, 256, 16, 0);
			} else
			if (IL.FR_Logs_Fireproof.equal(aInputs[0], T, T)) {
				if (UT.Fluids.equal(MT.Lubricant.liquid(U, T), aFluids[0]))	rRecipe = new Recipe(F, new ItemStack[] {UT.Stacks.amount_(1, aInputs[0])}, new ItemStack[] {UT.Stacks.create(IL.FR_Planks_Fireproof.getWithDamage(              6, UT.Stacks.meta(aInputs[0])), aInputs[0].getTagCompound()), dust.mat(MT.Wood, 1)}, null, null, new FluidStack[] {MT.Lubricant.liquid(U/200, T)}, ZL_FLUIDSTACK, 128, 16, 0); else
				if (UT.Fluids.equal(MT.Water.liquid(U, T), aFluids[0]))		rRecipe = new Recipe(F, new ItemStack[] {UT.Stacks.amount_(1, aInputs[0])}, new ItemStack[] {UT.Stacks.create(IL.FR_Planks_Fireproof.getWithDamage(NERFED_WOOD?4:5, UT.Stacks.meta(aInputs[0])), aInputs[0].getTagCompound()), dust.mat(MT.Wood, 2)}, null, null, new FluidStack[] {MT.Water    .liquid(U/ 50, T)}, ZL_FLUIDSTACK, 256, 16, 0);
			} else
			if (IL.FR_Planks.equal(aInputs[0], T, T)) {
				if (UT.Fluids.equal(MT.Lubricant.liquid(U, T), aFluids[0]))	rRecipe = new Recipe(F, new ItemStack[] {UT.Stacks.amount_(1, aInputs[0])}, new ItemStack[] {UT.Stacks.create(IL.FR_Slabs.getWithDamage(2, UT.Stacks.meta(aInputs[0])), aInputs[0].getTagCompound())}, null, null, new FluidStack[] {MT.Lubricant.liquid(U/1000, T)}, ZL_FLUIDSTACK,  24, 16, 0); else
				if (UT.Fluids.equal(MT.Water.liquid(U, T), aFluids[0]))		rRecipe = new Recipe(F, new ItemStack[] {UT.Stacks.amount_(1, aInputs[0])}, new ItemStack[] {UT.Stacks.create(IL.FR_Slabs.getWithDamage(2, UT.Stacks.meta(aInputs[0])), aInputs[0].getTagCompound())}, null, null, new FluidStack[] {MT.Water    .liquid(U/ 100, T)}, ZL_FLUIDSTACK,  96, 16, 0);
			} else
			if (IL.FR_Planks_Fireproof.equal(aInputs[0], T, T)) {
				if (UT.Fluids.equal(MT.Lubricant.liquid(U, T), aFluids[0]))	rRecipe = new Recipe(F, new ItemStack[] {UT.Stacks.amount_(1, aInputs[0])}, new ItemStack[] {UT.Stacks.create(IL.FR_Slabs_Fireproof.getWithDamage(2, UT.Stacks.meta(aInputs[0])), aInputs[0].getTagCompound())}, null, null, new FluidStack[] {MT.Lubricant.liquid(U/1000, T)}, ZL_FLUIDSTACK,  24, 16, 0); else
				if (UT.Fluids.equal(MT.Water.liquid(U, T), aFluids[0]))		rRecipe = new Recipe(F, new ItemStack[] {UT.Stacks.amount_(1, aInputs[0])}, new ItemStack[] {UT.Stacks.create(IL.FR_Slabs_Fireproof.getWithDamage(2, UT.Stacks.meta(aInputs[0])), aInputs[0].getTagCompound())}, null, null, new FluidStack[] {MT.Water    .liquid(U/ 100, T)}, ZL_FLUIDSTACK,  96, 16, 0);
			}
			
			if (rRecipe != null) rRecipe.mCanBeBuffered = F;
			
			return rRecipe;
		}
		
		@Override public boolean containsInput(ItemStack aStack) {return super.containsInput(aStack) || IL.FR_Logs.equal(aStack, T, T) || IL.FR_Logs_Fireproof.equal(aStack, T, T) || IL.FR_Planks.equal(aStack, T, T) || IL.FR_Planks_Fireproof.equal(aStack, T, T);}
*/	}
}